import unittest

from .codeConv import *
from .dirkBase import *


class TestCodeConv(unittest.TestCase):

    def test_1(self):
        pass
        # E文本编码转换(编码_UTF8编码("你好aaaa"), "", "utf-8")
        text = 编码_UTF8编码("大家好我是gbk编码你好aaaa")
        print(type(text))
        text2 = 文本编码转换(text, "", "gbk")
        print(type(text2))
        print(text2)
        print(编码_检查(text2))

        写到文件("gbk.txt",text2)
        print("--------------")

        text = 编码_GBK编码("大家好我是gbk编码你好aaaa")
        print(type(text))
        text2 = 文本编码转换(text, "", "utf-8")
        print(type(text2))
        print(text2)
        print(编码_检查(text2))
        写到文件("utf-8.txt",text2)

    def test_2(self):
        data = 编码_URL编码("你好ABC")
        print(data)



import unittest

from .dirkBase import *


class TestDirk(unittest.TestCase):

    def test_1(self):
        data = 取当前目录()
        self.assertNotEqual(data, "")
        print(data)

        data = 取运行目录()
        self.assertNotEqual(data, "")
        print(data)

        data = 写到文件("1.txt", b"abc")
        self.assertEqual(data, True)

        data = 写到文件("./a/1.txt", b"abc")
        self.assertEqual(data, False)

        data = 复制文件("1.txt", "2.txt")
        self.assertEqual(data, True)

        data = 复制文件("1.txt", "./a/2.txt")
        self.assertEqual(data, False)

        data = 移动文件("1.txt", "3.txt")
        self.assertEqual(data, True)

        data = 移动文件("1.txt", "./a/3.txt")
        self.assertEqual(data, False)

        data = 文件_目录文件列表()
        self.assertNotEqual(data, "")
        print(data)

        data = 文件_遍历子目录('./core')
        self.assertNotEqual(data, "")
        print(data)

        data = 创建目录('./b')
        self.assertEqual(data, True)
        print(data)

        data = 创建目录('./c/c/c/c')
        self.assertEqual(data, True)
        print(data)

        data = 创建目录('./c/bb')
        self.assertEqual(data, True)
        print(data)

        data = 读入文件('./2.txt')
        self.assertEqual(data, b'abc')
        print(data)

        data = 读入文件('./aaa.txt')
        self.assertEqual(data, b"")
        print(data)

        data = 删除文件('./2.txt')
        self.assertEqual(data, True)
        print(data)

        data = 删除文件('./3.txt')
        self.assertEqual(data, True)
        print(data)

        data = 删除文件('./4.txt')
        self.assertEqual(data, False)
        print(data)

        data = 删除目录('./b')
        self.assertEqual(data, True)
        print(data)
        data = 删除目录('./c', True)
        self.assertEqual(data, True)
        print(data)
        data = 删除目录('./aa/aa')
        self.assertEqual(data, False)
        print(data)

    def test_2(self):
        data = 文件_路径取扩展名('c:/a/1.jpg')
        self.assertEqual(data, ".jpg")
        print(data)

        data = 文件_取文件名('c:/a/1.jpg')
        self.assertEqual(data, "1.jpg")
        print(data)

        data = 文件_取目录('c:/a/1.jpg')
        self.assertEqual(data, "c:/a")
        print(data)

    def test_3(self):
        data = 写到文件("./1.txt", b"abc")
        self.assertEqual(data, True)

        data = 文件更名("./1.txt", "2.txt")
        self.assertEqual(data, True)

        data = 文件更名("./1.txt", "/a/a/2.txt")
        self.assertEqual(data, False)

    def test_4(self):
        data = 文件_路径取扩展名('c:/a/1.jpg')
        self.assertEqual(data, ".jpg")
        print(data)

        data = 文件_取文件名('c:/a/1.jpg')
        self.assertEqual(data, "1.jpg")
        print(data)

        data = 文件_取目录('c:/a/1.jpg')
        self.assertEqual(data, "c:/a")
        print(data)

        data = 改变目录('./core')
        self.assertEqual(data, True)
        print(data)

        data = 改变目录('./BBB')
        self.assertEqual(data, False)
        print(data)

        data = 取当前目录()
        self.assertEqual(data, 取运行目录() + "\core")
        print(data)

        # window 用不了
        # data = 改变当前进程目录('./coreUtil')
        # self.assertEqual(data, False)
        # print(data)
        #
        # data = 取当前目录()
        # self.assertEqual(data, 取运行目录() + "\ccoreUtil")
        # print(data)
        data = 写到文件("./1.txt", b"abc")
        self.assertEqual(data, True)
        data = 文件_检查权限("1.txt", 3)
        self.assertEqual(data, True)

        data = 文件_是否为绝对路径("1.txt")
        self.assertEqual(data, False)

        data = 文件_是否为绝对路径(取运行目录() + "/1.txt")
        self.assertEqual(data, True)

        data = 文件_是否为目录(取运行目录() + "/1.txt")
        self.assertEqual(data, False)

        data = 文件_是否为目录(取运行目录() + "/")
        self.assertEqual(data, True)

        data = 改变目录(取运行目录())
        data = 写到文件("./1.txt", b"abc")
        self.assertEqual(data, True)
        print(取运行目录() + "\\1.txt")
        data = 文件_是否为文件(取运行目录() + "\\1.txt")
        self.assertEqual(data, True)
        data = 文件_是否为文件(取运行目录() + "/")
        self.assertEqual(data, False)

        data = 文件是否存在(取运行目录())
        self.assertEqual(data, True)

        data = 文件_取文件大小(取运行目录() + "\\1.txt")
        self.assertEqual(data, 3)

    def test_5(self):
        data = 文件_获取文件信息(取运行目录() + "\\1.txt")
        print(data)
        data = 文件_修改文件时间(取运行目录() + "\\1.txt", (1621211781, 1621211781))
        print(data)
        data = 取文件访问时间(取运行目录() + "\\1.txt")
        print(data)
        data = 取文件创建时间(取运行目录() + "\\1.txt")
        print(data)
        data = 取文件修改时间(取运行目录() + "\\1.txt")
        print(data)

        文件_修改权限(取运行目录() + "\\1.txt", 0)
        data = 文件_检查权限("1.txt", 1)
        print(data)
        文件_修改权限(取运行目录() + "\\1.txt", 1)
        data = 文件_检查权限("1.txt", 2)
        print(data)

    def test_6(self):
        print(取运行目录())
        list = 文件_枚举(取运行目录(), ".py", False)
        for i in list:
            print(i)

        list = 文件_枚举(取运行目录(), ".py", True)
        for i in list:
            print(i)

    def test_7(self):
        print(取运行目录())
        list = 目录_枚举(取运行目录(), False)
        for i in list:
            print(i)

        list = 目录_枚举(取运行目录(), True)
        for i in list:
            print(i)

    def test_8(self):
        data = 文件_取扩展名("c:/1.txt")
        self.assertEqual(data, "txt")
        data = 文件_取父目录("c:/1.txt")
        self.assertEqual(data, "c:/")

    def test_9(self):
        print(取运行目录())
        #
        # data = 文件_写出(取运行目录() + r"\a\b\5.txt", b"bbb")
        # self.assertEqual(data, True)
        #
        # data = 文件_追加文本(取运行目录() + r"\a\b\1.txt", "bbb")
        # self.assertEqual(data, True)

        data = 文件_保存(取运行目录() + r"\a\b\m.txt", b"bbb")
        self.assertEqual(data, True)

        data = 读入文本(取运行目录() + r"\a\b\m.txt")
        print(data)


import unittest

from .threadingUtil import *
from .__init__ import *
from .asyncPoolGevent.asyncPoolGevent import *
# from .asyncPool.asyncPool import * 无法在测试用例中测试
# import asyncio

lock = 互斥锁()
lock2 = 递归锁()


def 工作线程(参数):
    lock.进入()
    print(取现行时间(), 参数, "启动线程了")
    lock.退出()


def 工作线程2(参数):
    print(取现行时间(), 参数, "无锁")


def 工作线程3(参数):
    lock2.进入()
    print(取现行时间(), 参数, "启动线程了 递归锁")
    lock2.退出()


def 工作线程4(参数):
    with 时间统计() as t:
        延时(取随机数(4, 4))
        # 延时(取随机数(5, 5))
        print(time.time(), t.取耗时(), 参数, "线程类测试")
        # print(取现行时间(), t.取耗时(), 参数, "线程类测试")


def 线程初始化(data):
    print("初始化", data, 取当前线程名称())


互斥锁 = 互斥锁()


def 任务函数(i):
    time.sleep(1)
    互斥锁.进入()
    print(i)
    互斥锁.退出()
    return "返回参数" + str(i)


def 任务完成(future):
    print("当前线程", 取当前线程名称())
    print("future", future.result())


class TestThreadingUtil(unittest.TestCase):

    def test_1(self):
        for i in range(10):
            data = 启动线程(工作线程, [i])

        延时(1)

    def test_4(self):
        for i in range(10):
            data = 启动线程(工作线程2, [i])
        延时(1)

    def test_5(self):
        for i in range(10):
            data = 启动线程(工作线程3, [i])
        延时(1)

    def test_2(self):
        lock = 互斥锁()
        lock.进入()
        lock.退出()

    def test_3(self):
        data = 网页_访问("https://www.baidu.com/")
        print(data.字节集.decode('utf-8'))

    #
    # def test_4(self):
    #     pool = 线程池(
    #         工作线程数量=2,
    #         队列数量=10,
    #     )
    #     for i in range(5):
    #         # print("投递任务1",i)
    #
    #         # 一直等待直到了有位置
    #         pool.投递任务(工作线程4, i)
    #         # 超时返回false
    #         # while pool.投递任务(工作线程4, i, 等待时间=None) == False:
    #         #     print("队列已满")
    #         print("投递任务", 取现行时间(), i)
    #     pool.等待()
    #     print("任务完成")

    def test_4(self):

        pool = 协程池(协程数量=9)
        任务列表 = []
        # "创建一批任务 一起运行等待完成"
        for i in range(11):
            print("创建任务", i)
            任务列表.append(pool.投递任务(工作线程4, i))
        pool.等待协程完成任务(任务列表)
        print("任务完成1")

        pool.等待()
        print("任务完成2")

    def test_10(self):
        # 创建任务直接运行不需要等待
        pool = 协程池(协程数量=9)
        for i in range(11):
            print("创建任务", i)
            pool.投递任务(工作线程4, i)
        pool.等待()

    def test_5(self):

        print("线程_取活动对象数", 线程_取活动对象数())
        print("线程_取当前线程", 线程_取当前线程())
        print("线程_取线程标识符", 线程_取线程标识符())
        print("线程_取所有活动对象", 线程_取所有活动对象())
        print("线程_取主线程", 线程_取主线程())

    def test_6(self):

        任务池 = 线程池(4, "pyefun", 线程初始化, [0])

        for url in range(10):
            future = 任务池.投递任务(任务函数, url)
            任务池.设置任务结束回调函数(future, 任务完成)

        任务池.等待()

        任务池.批量投递任务(任务函数, range(1, 12))  # map取代了for+submit
        任务池.等待()

    def test_8(self):
        pass
        # # 创建任务直接运行不需要等待
        # pool = 协程池Asyncio(协程数量=9, 线程池数量=10)
        # for i in range(11):
        #     print("创建任务", i)
        #     pool.投递任务(任务函数, i, 回调函数=任务完成)
        # pool.释放线程()
        # pool.等待()



"""
配置项 读取ini配置文件

"""
import configparser
from .__init__ import *

class 配置项(configparser.ConfigParser):
    def __init__(self):
        pass
        super().__init__()

    def 加载(self, 内容):

        self.read_string(内容)  # python3
        return self

    def 取所有项名(self, 节名):
        return self.options(节名)

    def 取所有项名和值(self, 节名):
        return self.items(节名)

    def 取所有节名(self):
        return self.sections()

    def 读配置项(self, 节名称, 配置项名称, 默认值=""):
        try:
            return self.get(节名称, 配置项名称)
        except:
            return 默认值

    def 写配置项(self, 节名称, 配置项名称, 值):
        if not self.has_section(节名称):
            self.add_section(节名称)
        return self.set(节名称, 配置项名称, 值)

    def 删除配置项(self, 节名称, 配置项名称):
        return self.remove_option(节名称, 配置项名称)

    def 删除节名(self, 节名称):
        return self.remove_section(节名称)

    def 取数据(self):
        fp = io.StringIO()
        self.write(fp)
        text = fp.getvalue()
        fp.close()
        return text

    def 取所有结果(self):
        字典 = 灵活字典()
        所有节名 = self.取所有节名()
        for 节名 in 所有节名:
            for v in self.取所有项名和值(节名):
                字典[节名][v[0]] = v[1]
        return 字典



import unittest

from .networkUtil import *


class TestNetworkUtil(unittest.TestCase):

    def test_1(self):
        data = 网页_取外网IP()
        print(data)

    def test_2(self):
        data = 网页_访问("https://www.125.la/")
        print(data.字节集.decode('utf-8'))

    def test_3(self):
        data = 网页_访问("https://www.baidu.com/")
        print(data.字节集.decode('utf-8'))



import wx
from .wxControl import *

class 窗口(wx.Frame, 公用方法):

    def 关闭(self):
        self.Close(True)

    @组件_异常检测
    def 置图标(self,图标路径):
        icon = wx.Icon(图标路径)
        self.SetIcon(icon)

    def 居中(self):
        '将窗口调整到屏幕中间'
        self.Centre()

    @组件_异常检测
    def 创建状态栏(self,项目,抓取器=True):#只能创建一个
        "项目格式为list，每个成员为tuple 格式(项目名,宽度)，如[('项目1',-1),('项目2',-1)],负数表示按比例分配"
        try:
            if self.状态栏:
                return False
        except:
            pass

        样式 = wx.STB_SHOW_TIPS|wx.STB_ELLIPSIZE_END|wx.FULL_REPAINT_ON_RESIZE
        样式 += wx.STB_SIZEGRIP  if 抓取器 else 0
        self.状态栏 = self.CreateStatusBar(style=样式)
        self.状态栏.SetFieldsCount(len(项目))
        self.状态栏.SetStatusWidths([x[1] for x in 项目])
        for x in range(len(项目)):
            self.状态栏.SetStatusText(项目[x][0], x)

    @组件_异常检测
    def 置状态栏项目宽度(self,宽度列表):
        '需要传入一个整数列表设置全部项目宽度'
        self.状态栏.SetStatusWidths(宽度列表)

    @组件_异常检测
    def 置状态栏项目文本(self,索引,文本):
        self.状态栏.SetStatusText(文本, 索引)

    @组件_异常检测
    def 取状态栏项目数(self):
        return self.状态栏.GetFieldsCount()

    @组件_异常检测
    def 取状态栏项目文本(self,索引):
        return self.状态栏.GetStatusText(索引)

    @组件_异常检测
    def 取状态栏项目宽度(self,索引):
        return self.状态栏.GetStatusWidth(索引)

    @组件_异常检测
    def 取状态栏项目样式(self,索引):
        return self.状态栏.GetStatusStyle(索引)

    @组件_异常检测
    def 置状态栏最小宽度(self,宽度):
        return self.状态栏.SetMinHeight(宽度)

    @组件_异常检测
    def 置状态栏项目数(self,项目数,宽度列表=[]):
        宽度列表 = 宽度列表 if 宽度列表 else [-1 for x in range(项目数)]
        return self.状态栏.SetFieldsCount(项目数,宽度列表)




import wx
from .wxControl import *

class 整数微调框(wx.SpinCtrl, 公用方法):
    pass

    def 取数值进制类型(self):
        '返回10或16，十进制或16进制'
        return self.GetBase()

    @组件_异常检测
    def 置数值进制类型(self,类型):
        '类型：10或16，十进制或16进制'
        return self.SetBase(类型)

    def 取最大值(self):
        return self.GetMax()

    def 取最小值(self):
        return self.GetMin()

    @组件_异常检测
    def 置最大值(self,数值):
        return self.SetMax(数值)

    @组件_异常检测
    def 置最小值(self,数值):
        return self.SetMin(数值)

    def 取数值范围(self):
        '获取微调的数值范围'
        return self.GetRange()

    @组件_异常检测
    def 置数值范围(self,最小值,最大值):
        '设置微调的数值范围'
        return self.SetRange(最小值,最大值)

    @组件_异常检测
    def 取当前数值(self):
        return self.GetValue()

    @组件_异常检测
    def 置当前数值(self,数值):
        return self.SetValue(数值)



from pyefun.arithmeticOperationBase import *
from pyefun.arrayActionBase import *
from pyefun.codeConv import *
from pyefun.dirkBase import *
from pyefun.public import *
from pyefun.stringBase import *
from pyefun.systemProcessingBase import *
from pyefun.timeBase import *
from pyefun.typeConv import *

from pyefun.cacheUtil import *
from pyefun.commonlyUtil import *
from pyefun.progiterUtil import *
from pyefun.stringUtil import *
from pyefun.timeUtil import *

from pyefun.regexpUtil import *

from pyefun.threadingUtil import *
from pyefun.networkUtil import *
from pyefun.configUtil import *
from pyefun.configEnvUtil import *
from pyefun.clockUtil import *

from pyefun.encoding.compress.egzip import *
from pyefun.encoding.compress.ezlib import *
from pyefun.encoding.ebinary.binary import *
from pyefun.encoding.ebase64.ebase64 import *
from pyefun.encoding.url.url import *

__version__ = '1.0.18'


# 类型转换
from .timeBase import *
import json

def 到文本(bytes):
    return str(bytes, encoding="utf-8")

def 到字节集(str):
    return bytes(str, encoding='utf-8')

def 到数值(val):
    return float(val)

def 到整数(val):
    return int(float(val))

def 到时间(str):
    return 创建日期时间(str)


def json到文本(obj):
    return json.dumps(obj)

def json解析(obj):
    return json.loads(obj)


import unittest

from .progiterUtil import *
import ubelt as ub


class TestOftenUtil(unittest.TestCase):

    def test_cache_1(self):
        cfgstr = 'repr-of-params-that-uniquely-determine-the-process'
        cacher = ub.Cacher('test_process', cfgstr)
        data = cacher.tryload()
        if data is None:
            myvar1 = 'result of expensive process'
            myvar2 = 'another result'
            data = myvar1, myvar2
            cacher.save(data)
        myvar1, myvar2 = data










    def test_3(self):
        pass
        data = {
            'custom_types': [slice(0, 1, None), 1 / 3],
            'nest_dict': {'k1': [1, 2, {3: {4, 5}}],
                          'key2': [1, 2, {3: {4, 5}}],
                          'key3': [1, 2, {3: {4, 5}}],
                          },
            'nest_dict2': {'k': [1, 2, {3: {4, 5}}]},
            'nested_tuples': [tuple([1]), tuple([2, 3]), frozenset([4, 5, 6])],
            'one_tup': tuple([1]),
            'simple_dict': {'spam': 'eggs', 'ham': 'jam'},
            'simple_list': [1, 2, 'red', 'blue'],
        }
        调试输出(data)


    def test_6(self):
        # a = ["a", "b", "c", "d", "e"]
        # prog = ProgIter(iter(a), desc="描述", show_times=False, verbose=3)
        # for n in prog:
        #     pass
        #     prog.set_extra('进度 num {}'.format(n))

        进度 = 进度显示(起始索引=0, 总数=100, 描述="枚举", 启用=False, 信息级别=3, 显示速率=True, 显示时间=True, 进度大小=10)
        进度.开始()
        for n in range(100):
            进度.下一步()
            # 延时(1)
            # print(n,进度.取进度())
        进度.完成()

    def test_7(self):
        p = 进度显示(range(100), 描述="进度", 信息级别=3)
        for n in p:
            pass
            p.附加输出("n=" + str(n))

    def test_8(self):
        for n in 进度显示(range(100), "进度", 信息级别=3):
            pass

    def test_9(self):
        for n in 进度显示(iter(["a", "b", "c", "d", "e", "f"]), "字母", 信息级别=3):
            pass
            print(n)

        p = 进度显示(iter(["a", "b", "c", "d", "e", "f"]), "字母", 信息级别=3)
        for n in p:
            pass
            p.附加输出("n=" + str(n))

    def test_10(self):
        for n in 进度显示(iter(["a", "b", "c", "d", "e", "f"]), "字母"):
            pass
            print(n)

    def test_101(self):
        for n in 进度显示(iter(["a", "b", "c", "d", "e", "f"]), "字母", 进度大小=2):
            pass
            print(n)

    def test_11(self):

        进度 = 进度显示(None, "下载", 100)
        进度.开始()
        for n in range(100):
            进度.下一步()
            # 延时(1)
            # print(n,进度.取进度())
        进度.完成()

        进度 = 进度显示(总数=100, 起始索引=0, 描述="进度")
        进度.开始()
        for n in range(100):
            进度.下一步()
            # 延时(1)
            # print(n,进度.取进度())
        进度.完成()



import wx
from .wxControl import *


class 按钮(wx.Button, 公用方法):
    pass
    @组件_异常检测
    def 置认证图标(self,显示=True):
        return self.SetAuthNeeded(显示)
    def 显示认证图标(self):
        return self.SetAuthNeeded()
    def 隐藏认证图标(self):
        return self.SetAuthNeeded(False)
    def 置顶层默认项(self):
        '设置后再窗口中按回车即可触发按钮点击事件'
        return self.SetDefault()

class Button(按钮, 公用方法):
    pass




import unittest

from .ezlib import *


class Testezlib(unittest.TestCase):

    def test_1(self):
        data = zlib压缩(b"1234567890")

        self.assertEqual(binascii.hexlify(data), b"789c33343236313533b7b03400000b2c020e")

        ddata = zlib解压(data)

        self.assertEqual(ddata, b"1234567890")

        # print('压缩的数据:长度 : {},内容 : {}'.format(len(data), ddata))



# 文本操作


# 调用格式： 〈整数型〉 取文本长度 （文本型 文本数据） - 系统核心支持库->文本操作
# 英文名称：len
# 取文本型数据的长度，不包含结束0。本命令为初级命令。
# 参数<1>的名称为“文本数据”，类型为“文本型（text）”。参数值指定欲检查其长度的文本数据。
#
# 操作系统需求： Windows、Linux
def 取文本长度(文本数据: str) -> int:
    return len(文本数据)


# 调用格式： 〈文本型〉 取文本左边 （文本型 欲取其部分的文本，整数型 欲取出字符的数目） - 系统核心支持库->文本操作
# 英文名称：left
# 返回一个文本，其中包含指定文本中从左边算起指定数量的字符。本命令为初级命令。
# 参数<1>的名称为“欲取其部分的文本”，类型为“文本型（text）”。
# 参数<2>的名称为“欲取出字符的数目”，类型为“整数型（int）”。
#
# 操作系统需求： Windows、Linux
def 取文本左边(欲取其部分的文本: str, 欲取出字符的数目: int) -> str:
    if 取文本长度(欲取其部分的文本) < 欲取出字符的数目:
        欲取出字符的数目 = 取文本长度(欲取其部分的文本)
    return 欲取其部分的文本[0:欲取出字符的数目]


# 调用格式： 〈文本型〉 取文本右边 （文本型 欲取其部分的文本，整数型 欲取出字符的数目） - 系统核心支持库->文本操作
# 英文名称：right
# 返回一个文本，其中包含指定文本中从右边算起指定数量的字符。本命令为初级命令。
# 参数<1>的名称为“欲取其部分的文本”，类型为“文本型（text）”。
# 参数<2>的名称为“欲取出字符的数目”，类型为“整数型（int）”。
#
# 操作系统需求： Windows、Linux
def 取文本右边(欲取其部分的文本: str, 欲取出字符的数目: int) -> str:
    l = 取文本长度(欲取其部分的文本)
    lpos = l - 欲取出字符的数目
    if lpos < 0:
        lpos = 0
    return 欲取其部分的文本[lpos:l]


# 调用格式： 〈文本型〉 取文本中间 （文本型 欲取其部分的文本，整数型 起始取出位置，整数型 欲取出字符的数目） - 系统核心支持库->文本操作
# 英文名称：mid
# 返回一个文本，其中包含指定文本中从指定位置算起指定数量的字符。本命令为初级命令。
# 参数<1>的名称为“欲取其部分的文本”，类型为“文本型（text）”。
# 参数<2>的名称为“起始取出位置”，类型为“整数型（int）”。1为首位置，2为第2个位置，如此类推。
# 参数<3>的名称为“欲取出字符的数目”，类型为“整数型（int）”。
#
# 操作系统需求： Windows、Linux
def 取文本中间(欲取其部分的文本, 起始取出位置, 欲取出字符的数目):
    欲取出字符的数目 = 起始取出位置 + 欲取出字符的数目
    if 取文本长度(欲取其部分的文本) < 欲取出字符的数目:
        欲取出字符的数目 = 取文本长度(欲取其部分的文本)
    return 欲取其部分的文本[起始取出位置: 欲取出字符的数目]


# 调用格式： 〈文本型〉 字符 （字节型 欲取其字符的字符代码） - 系统核心支持库->文本操作
# 英文名称：chr
# 返回一个文本，其中包含有与指定字符代码相关的字符。本命令为初级命令。
# 参数<1>的名称为“欲取其字符的字符代码”，类型为“字节型（byte）”。
#
# 操作系统需求： Windows、Linux
def 字符(字节型: bytes) -> str:
    return chr(字节型)


# 调用格式： 〈整数型〉 取代码 （文本型 欲取字符代码的文本，［整数型 欲取其代码的字符位置］） - 系统核心支持库->文本操作
# 英文名称：asc
# 返回文本中指定位置处字符的代码。如果指定位置超出文本长度，返回0。本命令为初级命令。
# 参数<1>的名称为“欲取字符代码的文本”，类型为“文本型（text）”。
# 参数<2>的名称为“欲取其代码的字符位置”，类型为“整数型（int）”，可以被省略。1为首位置，2为第2个位置，如此类推。如果本参数被省略，默认为首位置。
#
# 操作系统需求： Windows、Linux
def 取代码(欲取字符代码的文本: str) -> int:
    for item in 欲取字符代码的文本:
        return ord(item)
    return 0


# 调用格式： 〈整数型〉 寻找文本 （文本型 被搜寻的文本，文本型 欲寻找的文本，［整数型 起始搜寻位置］，逻辑型 是否不区分大小写） - 系统核心支持库->文本操作
# 英文名称：InStr
# 返回一个整数值，指定一文本在另一文本中最先出现的位置，位置值从 1 开始。如果未找到，返回-1。本命令为初级命令。
# 参数<1>的名称为“被搜寻的文本”，类型为“文本型（text）”。
# 参数<2>的名称为“欲寻找的文本”，类型为“文本型（text）”。
# 参数<3>的名称为“起始搜寻位置”，类型为“整数型（int）”，可以被省略。位置值从 1 开始。如果本参数被省略，默认为 1 。
# 参数<4>的名称为“是否不区分大小写”，类型为“逻辑型（bool）”，初始值为“假”。为真不区分大小写，为假区分。
#
# 操作系统需求： Windows、Linux
# 找不到返回-1
def 寻找文本(被搜寻的文本: str, 欲寻找的文本: str, 起始搜寻位置=None, 结束的位置=None) -> int:
    return 被搜寻的文本.find(欲寻找的文本, 起始搜寻位置, 结束的位置)


def 倒找文本(被搜寻的文本: str, 欲寻找的文本: str, 起始搜寻位置=None, 结束的位置=None) -> int:
    return 被搜寻的文本.rfind(欲寻找的文本, 起始搜寻位置, 结束的位置)


# 调用格式： 〈文本型〉 到大写 （文本型 欲变换的文本） - 系统核心支持库->文本操作
# 英文名称：UCase
# 将文本中的小写英文字母变换为大写，返回变换后的结果文本。本命令为初级命令。
# 参数<1>的名称为“欲变换的文本”，类型为“文本型（text）”。
#
# 操作系统需求： Windows、Linux
def 到大写(欲变换的文本: str) -> str:
    return 欲变换的文本.upper()


def 到小写(欲变换的文本: str) -> str:
    return 欲变换的文本.casefold()


def 到全角(欲变换的文本: str) -> str:
    rstring = ""
    for uchar in 欲变换的文本:
        inside_code = ord(uchar)
        if inside_code == 32:  # 半角空格直接转化
            inside_code = 12288
        elif inside_code >= 32 and inside_code <= 126:  # 半角字符（除空格）根据关系转化
            inside_code += 65248

        rstring += chr(inside_code)
    return rstring


def 到半角(欲变换的文本: str) -> str:
    """全角转半角"""
    ss = []
    for s in 欲变换的文本:
        rstring = ""
        for uchar in s:
            inside_code = ord(uchar)
            if inside_code == 12288:  # 全角空格直接转换
                inside_code = 32
            elif inside_code >= 65281 and inside_code <= 65374:
                # 全角字符（除空格）根据关系转化
                inside_code -= 65248

            rstring += chr(inside_code)

        ss.append(rstring)
    return ''.join(ss)


# 调用格式： 〈文本型〉 删首空 （文本型 欲删除空格的文本） - 系统核心支持库->文本操作
# 英文名称：LTrim
# 返回一个文本，其中包含被删除了首部全角或半角空格的指定文本。本命令为初级命令。
# 参数<1>的名称为“欲删除空格的文本”，类型为“文本型（text）”。
#
# 操作系统需求： Windows、Linux
def 删首空(欲删除空格的文本: str) -> str:
    return 欲删除空格的文本.lstrip()


def 删尾空(欲删除空格的文本: str) -> str:
    return 欲删除空格的文本.rstrip()


def 删首尾空(欲删除空格的文本: str) -> str:
    return 删尾空(删首空(欲删除空格的文本))


def 删全部空(欲删除空格的文本: str) -> str:
    return 欲删除空格的文本.replace(" ", "")


# 调用格式： 〈文本型〉 子文本替换 （文本型 欲被替换的文本，文本型 欲被替换的子文本，［文本型 用作替换的子文本］，［整数型 进行替换的起始位置］，［整数型 替换进行的次数］，逻辑型 是否区分大小写） - 系统核心支持库->文本操作
# 英文名称：RpSubText
# 返回一个文本，该文本中指定的子文本已被替换成另一子文本，并且替换发生的次数也是被指定的。本命令为初级命令。
# 参数<1>的名称为“欲被替换的文本”，类型为“文本型（text）”。
# 参数<2>的名称为“欲被替换的子文本”，类型为“文本型（text）”。
# 参数<3>的名称为“用作替换的子文本”，类型为“文本型（text）”，可以被省略。如果本参数被省略，默认为空文本。
# 参数<4>的名称为“进行替换的起始位置”，类型为“整数型（int）”，可以被省略。参数值指定被替换子文本的起始搜索位置。如果省略，默认从 1 开始。
# 参数<5>的名称为“替换进行的次数”，类型为“整数型（int）”，可以被省略。参数值指定对子文本进行替换的次数。如果省略，默认进行所有可能的替换。
# 参数<6>的名称为“是否区分大小写”，类型为“逻辑型（bool）”，初始值为“真”。为真区分大小写，为假不区分。
#
# 操作系统需求： Windows、Linux
def 子文本替换(欲被替换的文本: str, 欲被替换的子文本: str, 用作替换的子文本: str, 替换进行的次数=-1) -> str:
    return 欲被替换的文本.replace(欲被替换的子文本, 用作替换的子文本, 替换进行的次数)


# 调用格式： 〈文本型〉 取空白文本 （整数型 重复次数） - 系统核心支持库->文本操作
# 英文名称：space
# 返回具有指定数目半角空格的文本。本命令为初级命令。
# 参数<1>的名称为“重复次数”，类型为“整数型（int）”。
#
# 操作系统需求： Windows、Linux
def 取空白文本(重复次数: int) -> str:
    return 取重复文本(重复次数, " ")


# 调用格式： 〈文本型〉 取重复文本 （整数型 重复次数，文本型 待重复文本） - 系统核心支持库->文本操作
# 英文名称：string
# 返回一个文本，其中包含指定次数的文本重复结果。本命令为初级命令。
# 参数<1>的名称为“重复次数”，类型为“整数型（int）”。
# 参数<2>的名称为“待重复文本”，类型为“文本型（text）”。该文本将用于建立返回的文本。如果为空，将返回一个空文本。
#
# 操作系统需求： Windows、Linux
def 取重复文本(重复次数: int, 待重复文本: str) -> str:
    str = ""
    for _ in range(重复次数):
        str = str + 待重复文本
    return str


# 调用格式： 〈文本型数组〉 分割文本 （文本型 待分割文本，［文本型 用作分割的文本］，［整数型 要返回的子文本数目］） - 系统核心支持库->文本操作
# 英文名称：split
# 将指定文本进行分割，返回分割后的一维文本数组。本命令为初级命令。
# 参数<1>的名称为“待分割文本”，类型为“文本型（text）”。如果参数值是一个长度为零的文本，则返回一个空数组，即没有任何成员的数组。
# 参数<2>的名称为“用作分割的文本”，类型为“文本型（text）”，可以被省略。参数值用于标识子文本边界。如果被省略，则默认使用半角逗号字符作为分隔符。如果是一个长度为零的文本，则返回的数组仅包含一个成员，即完整的“待分割文本”。
# 参数<3>的名称为“要返回的子文本数目”，类型为“整数型（int）”，可以被省略。如果被省略，则默认返回所有的子文本。
#
# 操作系统需求： Windows、Linux
def 分割文本(待分割文本: str, 用作分割的文本: str = ",", 要返回的子文本数目: int = -1) -> dict:
    return 待分割文本.split(用作分割的文本, 要返回的子文本数目)



"""
组件公用方法

"""
import wx.lib.agw.floatspin as floatspin
import wx.lib.agw.hyperlink as hyperlink
import wx.lib.agw.gradientbutton as gradientbutton
import wx.lib.buttons as lib_button
import wx.adv
import wx
import datetime
import traceback


def 组件_异常检测(function):
    '装饰器'

    def box(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except:
            print(function.__name__, "函数发生异常")
            print("错误发生时间：", str(datetime.datetime.now()))
            print("错误的详细情况：", traceback.format_exc())

    return box


class 公用方法(wx.Control):
    def 绑定事件(self, event, handler, source=None, id=-1, id2=-1):  # reliably restored by inspect
        self.Bind(event, handler, source=source, id=id, id2=id2)

    def 置标题(self, 内容):
        self.SetName(内容)

    def 取窗口句柄(self):
        return self.GetHandle()

    @组件_异常检测
    def 取组件名称(self):
        return self.GetName()

    @组件_异常检测
    def 取标记ID(self):
        return self.GetId()

    @组件_异常检测
    def 取左边顶边(self):
        return self.GetPosition()

    @组件_异常检测
    def 取左边(self):
        return self.GetPosition()[0]

    @组件_异常检测
    def 取顶边(self):
        return self.GetPosition()[1]

    @组件_异常检测
    def 取宽度高度(self):
        return self.GetSize()

    @组件_异常检测
    def 取宽度高度2(self):
        return self.GetClientSize()

    @组件_异常检测
    def 取宽高3(self):
        '将窗口的最佳大小合并为最小大小'
        return self.GetEffectiveMinSize()

    @组件_异常检测
    def 取宽度(self):
        return self.GetSize()[0]

    @组件_异常检测
    def 取高度(self):
        return self.GetSize()[1]

    @组件_异常检测
    def 取祖组件(self):
        return self.GetGrandParent()

    @组件_异常检测
    def 取桌面相对坐标(self):
        "取组件左边跟顶边相对于桌面的坐标位置"
        return self.GetScreenPosition()

    @组件_异常检测
    def 取桌面相对坐标2(self):
        "取组件左边跟顶边相对于桌面的坐标位置"
        return self.ClientToScreen(0, 0)

    @组件_异常检测
    def 取窗口相对屏幕矩形(self):
        '出错返回False,返回窗口在屏幕坐标中的位置，无论该窗口是子窗口还是顶级窗口,格式:(相对于屏幕的左边,相对于屏幕的顶边,组件宽度,组件高度)'
        return self.GetScreenRect()

    @组件_异常检测
    def 取背景颜色(self):
        '返回窗口的背景色,格式:(240, 240, 240, 255)'
        return self.GetBackgroundColour()

    @组件_异常检测
    def 取可设置的最小尺寸(self):
        '返回窗口的最佳可接受最小尺寸, 返回格式: (宽度, 高度)，高度不包含标题栏高度'
        return self.GetBestSize()

    @组件_异常检测
    def 取可设置的最大尺寸(self):
        '返回窗口的最佳可接受最大尺寸, 返回格式: (宽度, 高度)，高度不包含标题栏高度'
        return self.GetBestVirtualSize()

    @组件_异常检测
    def 取主题样式(self):
        '样式：0.默认背景样式值,1.使用由系统或当前主题确定的默认背景,2.指示仅在用户定义的EVT_PAINT处理程序中擦除背景,3.无介绍,4.表示未擦除窗口背景，从而使父窗口得以显示'
        return self.GetBackgroundStyle()

    @组件_异常检测
    def 取边框样式(self):
        return self.GetBorder()

    @组件_异常检测
    def 取额外样式(self):
        return self.GetExtraStyle()

    @组件_异常检测
    def 取字体及颜色(self):
        '返回: 字体,背景颜色,前景颜色，#(<wx._core.Font object at 0x000002140997DB88>, wx.Colour(240, 240, 240, 255), wx.Colour(0, 0, 0, 255))'
        结果 = self.GetClassDefaultAttributes()
        return 结果.font, 结果.colBg, 结果.colFg

    @组件_异常检测
    def 取矩形(self):
        '返回窗口矩形：(左边,顶边,宽度,高度)'
        return self.GetRect()

    @组件_异常检测
    def 取工作区矩形(self):
        return self.GetClientRect()

    @组件_异常检测
    def 取字体(self):
        return self.GetFont()

    @组件_异常检测
    def 取字体高度(self):
        '返回此窗口的字符高度'
        return self.GetCharHeight()

    @组件_异常检测
    def 取字体平均宽度(self):
        '返回此窗口的平均字符宽度'
        return self.GetCharWidth()

    @组件_异常检测
    def 取前景色(self):
        return self.GetForegroundColour()

    @组件_异常检测
    def 取标题(self):
        return self.GetLabel()

    @组件_异常检测
    def 取标题T(self):
        return self.GetTitle()

    @组件_异常检测
    def 取内容(self):
        return self.GetValue()

    @组件_异常检测
    def 取工作区最小宽高(self):
        '返回窗口的工作区的最小大小，这向sizer布局机制指示这是其工作区的最小所需大小'
        return self.GetMinClientSize()

    @组件_异常检测
    def 取最小宽高(self):
        '返回窗口的最小大小，这向sizer布局机制指示这是最小所需大小'
        return self.GetMinSize()

    @组件_异常检测
    def 取下一个组件(self):
        '返回下一个组件的对象，即按TAB键切换到的下一个组件'
        return self.GetNextSibling()

    @组件_异常检测
    def 取下上一个组件(self):
        '返回上一个组件的对象，即按TAB键切换到的上一个组件'
        return self.GetPrevSibling()

    @组件_异常检测
    def 取父窗口(self):
        '返回父窗口对象'
        return self.GetParent()

    @组件_异常检测
    def 取顶级窗口(self):
        return self.GetTopLevelParent()

    @组件_异常检测
    def 取虚拟宽高(self):
        '这将获取窗口的虚拟大小,它返回窗口的客户端大小，但是在调用SetVirtualSize 它之后，将返回使用该方法设置的大小'
        return self.GetVirtualSize()

    @组件_异常检测
    def 取内置滚动条缩略图大小(self, 方向):
        '返回内置滚动条的缩略图大小,方向：4.横向滚动条 8.纵向滚动条'
        return self.GetScrollThumb(方向)

    @组件_异常检测
    def 取内置滚动条范围(self, 方向):
        '返回内置滚动条范围,方向：4.横向滚动条 8.纵向滚动条'
        return self.GetScrollRange(方向)

    @组件_异常检测
    def 取内置滚动条位置(self, 方向):
        '返回内置滚动条的位置,方向：4.横向滚动条 8.纵向滚动条'
        return self.GetScrollPos(方向)

    @组件_异常检测
    def 置组件名称(self, 名称):
        return self.SetName(名称)

    def 置标记ID(self):
        return self.SetId()

    @组件_异常检测
    def 置左边顶边(self, 左边, 顶边):
        return self.Move(左边, 顶边)

    @组件_异常检测
    def 置左边顶边2(self, 左边, 顶边):
        return self.SetPosition(左边, 顶边)

    @组件_异常检测
    def 置左边(self, 左边):
        return self.Move(左边, self.GetPosition()[1])

    @组件_异常检测
    def 置顶边(self, 顶边):
        return self.Move(self.GetPosition()[0], 顶边)

    @组件_异常检测
    def 置左边2(self, 左边):
        return self.SetPosition(左边, self.GetPosition()[1])

    @组件_异常检测
    def 置顶边2(self, 顶边):
        return self.SetPosition(self.GetPosition()[0], 顶边)

    @组件_异常检测
    def 置宽度高度(self, 宽度, 高度):
        return self.GetSize((宽度, 高度))

    @组件_异常检测
    def 置宽度高度2(self, 宽度, 高度):
        return self.SetInitialSize((宽度, 高度))

    @组件_异常检测
    def 置宽度(self, 宽度):
        return self.SetInitialSize((宽度, self.GetSize()[1]))

    @组件_异常检测
    def 置高度(self, 高度):
        return self.SetInitialSize((self.GetSize()[0], 高度))

    @组件_异常检测
    def 置标题(self, 标题):
        return self.SetLabel(标题)

    @组件_异常检测
    def 置标题T(self, 标题):
        return self.SetTitle(标题)

    @组件_异常检测
    def 置内容(self, 内容):
        return self.SetValue(内容)

    @组件_异常检测
    def 置拖放权限(self, 拖放=True):
        "设置是否允许接收拖放文件"
        return self.DragAcceptFiles(拖放)

    @组件_异常检测
    def 置工作区宽高(self, 宽度, 高度):
        '出设置组件工作区的宽高(不包含边框,标题栏的宽高)'
        return self.SetClientSize(宽度, 高度)

    @组件_异常检测
    def 置背景颜色(self, 颜色):
        return self.SetBackgroundColour(颜色)

    @组件_异常检测
    def 单独置背景颜色(self, 颜色):
        '设置窗口的背景色，但防止其被该窗口的子级继承'
        return self.SetOwnBackgroundColour(颜色)

    @组件_异常检测
    def 置前景颜色(self, 颜色):
        return self.SetForegroundColour(颜色)

    @组件_异常检测
    def 单独置前景颜色(self, 颜色):
        '设置窗口的前景色，但防止其被该窗口的子级继承'
        return self.SetOwnForegroundColour(颜色)

    @组件_异常检测
    def 置最大宽高(self, 宽度, 高度):
        '设置整个窗口最大可设置的尺寸'
        return self.SetMaxSize((宽度, 高度))

    @组件_异常检测
    def 置最小宽高(self, 宽度, 高度):
        '设置整个窗口最小可设置的尺寸'
        return self.SetMinSize((宽度, 高度))

    @组件_异常检测
    def 置工作区最大宽高(self, 宽度, 高度):
        '设置工作区最大可设置的尺寸'
        return self.SetMaxClientSize((宽度, 高度))

    @组件_异常检测
    def 置工作区最小宽高(self, 宽度, 高度):
        '设置工作区最小可设置的尺寸'
        return self.SetMinClientSize((宽度, 高度))

    @组件_异常检测
    def 置虚拟宽高(self, 宽度, 高度):
        '设置窗口的虚拟大小（以像素为单位）'
        return self.SetVirtualSize((宽度, 高度))

    @组件_异常检测
    def 置透明度(self, 透明度):
        '设置窗口与透明度,范围0-255(0.完全透明,255完全不透明)'
        return self.SetTransparent(透明度)

    @组件_异常检测
    def 置主题样式(self, 样式):
        '窗口样式:0.默认(可擦除背景),1.跟随系统主题,2.指示仅在用户定义的EVT_PAINT处理程序中擦除背景,3.表示未擦除窗口背景，从而使父窗口得以显示,4.无描述。'
        return self.SetBackgroundStyle(样式)

    @组件_异常检测
    def 置窗口样式(self, 样式):
        return self.SetWindowStyleFlag(样式)

    @组件_异常检测
    def 置鼠标样式(self, 样式):
        '''

        出错返回False,样式:
        0:无描述
        1:标准箭头光标。
        2:指向右侧的标准箭头光标。
        3:靶心光标。
        4:矩形字符光标。
        5:十字光标。
        6:手形光标。
        7:工字梁光标（垂直线）。
        8:表示鼠标左键按下。
        9:放大镜图标。
        10:表示按下中间按钮的鼠标。
        11:不可输入的符号光标。
        12:画笔光标。
        13:铅笔光标。
        14:指向左的光标。
        15:指向右的光标。
        16:箭头和问号。
        17:表示按下了右键的鼠标。
        18:调整大小的光标指向NE-SW。
        19:调整大小的光标指向N-S。
        20:调整大小的光标指向NW-SE。
        21:调整大小的光标指向W-E。
        22:一般大小的游标。
        23:Spraycan游标。
        24:等待光标。
        25:监视光标。
        26:透明光标。
        27:带有标准箭头的等待光标。
        28:无描述。

        '''
        return self.SetCursor(wx.Cursor(样式))

    @组件_异常检测
    def 取鼠标样式(self):
        return self.GetCursor()

    @组件_异常检测
    def 置字体(self, 字体):
        return self.SetFont(字体)

    @组件_异常检测
    def 置字体2(self, 字体名, 大小, 粗细, 下划线):
        return self.SetFont(wx.Font(大小, wx.DEFAULT, wx.NORMAL, 粗细, 下划线, 字体名))

    @组件_异常检测
    def 置文本颜色(self, 颜色):
        return self.SetForegroundColour(颜色)

    @组件_异常检测
    def 置组件顺序_上(self, 上一个组件):
        '设置使用TAB键切换组件时的切换顺序，从上一个组件按下TAB键后跳转到单前组件'
        return self.MoveAfterInTabOrder(上一个组件)

    @组件_异常检测
    def 置组件顺序_下(self, 下一个组件):
        '设置使用TAB键切换组件时的切换顺序，从在单前组件按下TAB键后切换到下一个组件'
        return self.MoveBeforeInTabOrder(下一个组件)

    @组件_异常检测
    def 置滚动条属性(self, 方向, 位置, 可见大小, 最大位置, 重绘):
        '''
        设置滚动条位置,方向可选4或8,重绘True或False(设置内置滚动条之一的位置)
        假设您希望使用相同的字体显示50行文本。窗口的大小设置为一次只能看到16行。您将使用：
        self.SetScrollbar(wx.VERTICAL, 0, 16, 50)
        '''
        return self.SetScrollbar(方向, 位置, 可见大小, 最大位置, 重绘)

    @组件_异常检测
    def 置滚动条位置(self, 方向, 位置, 重绘):
        '设置滚动条位置,方向可选4或8,重绘True或False(设置内置滚动条之一的位置)'
        return self.SetScrollPos(方向, 位置, 重绘)

    @组件_异常检测
    def 销毁(self):
        return self.Destroy()

    @组件_异常检测
    def 销毁2(self):
        '官方解释：计划在不久的将来销毁该窗口,每当销毁可能发生得太早时（例如，当该窗口或其子级仍在事件队列中等待时），都应使用此方法'
        return self.DestroyLater()

    @组件_异常检测
    def 销毁子窗口(self):
        return self.DestroyChildren()

    @组件_异常检测
    def 禁用2(self):
        return self.Disable()

    @组件_异常检测
    def 禁用(self, 禁用=True):
        "True为禁用组件，False为恢复组件使用"
        return self.Enable(not 禁用)

    @组件_异常检测
    def 禁止重画(self):
        return self.Freeze()

    @组件_异常检测
    def 允许重画(self):
        return self.Thaw()

    @组件_异常检测
    def 移动(self, 左边=-1, 顶边=-1, 宽度=-1, 高度=-1):
        '调整移动窗口的左边跟顶边位置并重新设置宽度跟高度,不想调整的填-1'
        return self.SetSize(左边, 顶边, 宽度, 高度)

    @组件_异常检测
    def 刷新重绘(self, 删除背景=False):
        '导致GTK1重新绘制此窗口及其所有子级（除非未实现此子级）'
        return self.Refresh(删除背景)

    @组件_异常检测
    def 刷新重绘2(self):
        '调用此方法将立即重新绘制窗口的无效区域及其所有子级的对象（通常仅在控制流返回事件循环时才发生）'
        return self.Update()

    @组件_异常检测
    def 遍历下级组件(self):
        '遍历组件下的子级组件,返回在WindowList 列表里'
        return self.GetChildren()

    @组件_异常检测
    def 弹出菜单(self, 菜单, 左边=0, 顶边=0):
        '此函数在此窗口中的给定位置显示一个弹出菜单，并返回所选的ID'
        return self.GetPopupMenuSelectionFromUser(菜单, 左边, 顶边)

    @组件_异常检测
    def 移动鼠标(self, x, y):
        '移动鼠标到组件内的指定位置'
        return self.WarpPointer(x, y)

    @组件_异常检测
    def 是否有焦点(self):
        return self.HasFocus()

    @组件_异常检测
    def 是否有滚动条(self, 方向):
        '返回此窗口当前是否具有该方向的滚动条,方向：4.横向滚动条 8.纵向滚动条'
        return self.HasScrollbar(方向)

    @组件_异常检测
    def 是否透明(self):
        return self.HasTransparentBackground()

    @组件_异常检测
    def 显示或隐藏(self, 是否显示=True):
        '显示或隐藏窗口'
        return self.Show(是否显示)

    @组件_异常检测
    def 隐藏(self):
        return self.Show(False)

    @组件_异常检测
    def 隐藏2(self):
        return self.Hide()

    @组件_异常检测
    def 隐藏_带特效(self, 效果, 效果时长):
        '''
        出错返回False,此功能可隐藏一个窗口并使用特殊的视觉效果
        效果：0.无效果，1.向左滚动窗口，2.向右滚动窗口，3.将窗口滚动到顶部，4.将窗口滚动到底部，5.向左滑动窗口，6.向右滑动窗口，7.将窗口滑动到顶部，8.将窗口滑动到底部，9.淡入或淡出效果，10.扩大或崩溃的作用
        效果时长：单位毫秒
        '''
        return self.HideWithEffect(效果, 效果时长)

    @组件_异常检测
    def 显示(self):
        return self.Show(True)

    @组件_异常检测
    def 显示_带特效(self, 效果, 效果时长):
        '''
        出错返回False,此功能可隐藏一个窗口并使用特殊的视觉效果
        效果：0.无效果，1.向左滚动窗口，2.向右滚动窗口，3.将窗口滚动到顶部，4.将窗口滚动到底部，5.向左滑动窗口，6.向右滑动窗口，7.将窗口滑动到顶部，8.将窗口滑动到底部，9.淡入或淡出效果，10.扩大或崩溃的作用
        效果时长：单位毫秒
        '''
        return self.ShowWithEffect(效果, 效果时长)

    @组件_异常检测
    def 是否继承父窗口背景色(self):
        return self.InheritsBackgroundColour()

    @组件_异常检测
    def 是否继承父窗口前景色(self):
        return self.InheritsForegroundColour()

    @组件_异常检测
    def 重置缓存最佳大小(self):
        '出错返回False,重置缓存的最佳大小值，以便下次需要时重新计算'
        return self.InvalidateBestSize()

    @组件_异常检测
    def 是否正在销毁(self):
        '出错返回False,此窗口是否正在销毁中'
        return self.IsBeingDeleted()

    @组件_异常检测
    def 是否禁用(self):
        return not self.IsEnabled()

    @组件_异常检测
    def 是否可获取焦点(self):
        return self.IsFocusable()

    @组件_异常检测
    def 是否为上级窗口(self, 待判断组件):
        return self.IsDescendant(待判断组件)

    @组件_异常检测
    def 是否禁止重画(self):
        return self.IsFrozen()

    @组件_异常检测
    def 是否隐藏(self):
        '判断是否调用命令隐藏了窗口,最小化,遮挡,不算隐藏'
        return self.IsShown()

    @组件_异常检测
    def 是否允许透明(self):
        return self.CanSetTransparent()

    @组件_异常检测
    def 是否显示在屏幕上(self):
        return self.IsShownOnScreen()

    @组件_异常检测
    def 是否启用(self):
        '是否从本质上启用了此窗口，False否则返回'
        return self.IsThisEnabled()

    @组件_异常检测
    def 是否为顶级窗口(self):
        return self.IsTopLevel()

    @组件_异常检测
    def 重绘指定区域(self, 矩形=(0, 0, 0, 0), 擦除背景=True):
        '重绘指定矩形的内容：仅对其内部的区域进行重绘'
        return self.RefreshRect(矩形, 擦除背景)

    @组件_异常检测
    def 修改父级窗口(self, 新父级组件):
        '即该窗口将从其当前父窗口中移除加入到新的父级窗口下'
        return self.Reparent(新父级组件)

    @组件_异常检测
    def 桌面坐标转窗口内坐标(self, x, y):
        '从屏幕转换为客户端窗口内工作区坐标,'
        return self.ScreenToClient(x, y)

    @组件_异常检测
    def 到最顶层(self):
        '调整显示顺序'
        self.Raise()

    @组件_异常检测
    def 到最底层(self):
        '调整显示顺序'
        self.Lower()

    @组件_异常检测
    def 是否已设置背景色(self):
        return self.UseBackgroundColour()

    @组件_异常检测
    def 是否已设置前景色(self):
        return self.UseForegroundColour()

    @组件_异常检测
    def 向上滚动(self):
        '与ScrollLines （-1）相同,返回True是否滚动窗口，False如果窗口已经在顶部，则什么也不做'
        return self.LineUp()

    @组件_异常检测
    def 向下滚动(self):
        '与ScrollLines （1）相同，返回True是否滚动窗口，False如果窗口已经在底部，则什么也不做'
        return self.LineDown()

    @组件_异常检测
    def 是否始终显示滚动条(self, 方向):
        '判断滚动条是否始终显示,方向：4.横向滚动条 8.纵向滚动条'
        return self.IsScrollbarAlwaysShown(方向)

    @组件_异常检测
    def 滚动_页(self, 滚动页数=1):
        '滚动页数:向上滚动1次为-1,向下为1'
        return self.ScrollPages(滚动页数)

    @组件_异常检测
    def 滚动_行(self, 滚动行数=1):
        '出错返回False,滚动行数:向上滚动1次为-1,向下为1'
        return self.ScrollLines(滚动行数)

    @组件_异常检测
    def 是否使用系统主题设置背景(self):
        '窗口是否使用系统主题绘制其背景'
        return self.GetThemeEnabled()

    @property
    def 高度(self):
        return self.取高度()

    @property
    def 左边(self):
        return self.取左边()

    @property
    def 宽度(self):
        return self.取宽度()

    @property
    def 顶边(self):
        return self.取顶边()

    @高度.setter
    def 高度(self, value):
        self.置高度(value)

    @宽度.setter
    def 宽度(self, value):
        self.置宽度(value)

    @左边.setter
    def 左边(self, value):
        self.置左边(value)

    @顶边.setter
    def 顶边(self, value):
        self.置顶边(value)

    @property
    def 可视(self):
        return self.是否隐藏()

    @可视.setter
    def 可视(self, value):
        self.显示或隐藏(value)

    @property
    def 禁止(self):
        return self.是否禁用()

    @禁止.setter
    def 禁止(self, value):
        self.禁用(value)

    @property
    def 鼠标指针(self):
        return self.取鼠标样式()

    @鼠标指针.setter
    def 鼠标指针(self, value):
        '''
        鼠标指针_

        wx.StockCursor(鼠标样式)
        详细文档：https://wxpython.org/Phoenix/docs/html/wx.StockCursor.enumeration.html


        出错返回False,样式:
        0:无描述
        1:标准箭头光标。
        2:指向右侧的标准箭头光标。
        3:靶心光标。
        4:矩形字符光标。
        5:十字光标。
        6:手形光标。
        7:工字梁光标（垂直线）。
        8:表示鼠标左键按下。
        9:放大镜图标。
        10:表示按下中间按钮的鼠标。
        11:不可输入的符号光标。
        12:画笔光标。
        13:铅笔光标。
        14:指向左的光标。
        15:指向右的光标。
        16:箭头和问号。
        17:表示按下了右键的鼠标。
        18:调整大小的光标指向NE-SW。
        19:调整大小的光标指向N-S。
        20:调整大小的光标指向NW-SE。
        21:调整大小的光标指向W-E。
        22:一般大小的游标。
        23:Spraycan游标。
        24:等待光标。
        25:监视光标。
        26:透明光标。
        27:带有标准箭头的等待光标。
        28:无描述。

        '''
        return self.置鼠标样式(value)

    @property
    def 边框(self):
        return self.取边框样式()

    @边框.setter
    def 边框(self, value):
        pass
        # 等待编写
        return self.置窗口样式(value)

    @property
    def 标题(self):
        return self.取标题()

    @标题.setter
    def 标题(self, value):
        return self.置标题(value)

    @property
    def 名称(self):
        return self.取组件名称()

    @名称.setter
    def 名称(self, value):
        return self.置组件名称(value)

    @property
    def 备注(self):
        return self.备注

    @备注.setter
    def 备注(self, value):
        self.备注 = value

    @property
    def 标记(self):
        return self.标记

    @标记.setter
    def 标记(self, value):
        self.标记 = value

    @property
    def 文本颜色(self):
        return self.文本颜色

    @文本颜色.setter
    def 文本颜色(self, value):
        self.置文本颜色(value)
        self.文本颜色 = value

    @property
    def 背景颜色(self):
        return self.取背景颜色()

    @背景颜色.setter
    def 背景颜色(self, value):
        return self.置背景颜色(value)

    @property
    def 窗口样式(self):
        return self.取主题样式()

    @窗口样式.setter
    def 窗口样式(self, value):
        return self.置主题样式(value)



import wx
from .wxControl import *

class 列表框(wx.ListBox, 公用方法):
    pass
    @组件_异常检测
    def 取消指定选中项(self,索引):
        '在列表框中取消选择一个项目。'
        return self.Deselect(索引)

    @组件_异常检测
    def 保证显示(self,索引):
        '确保当前显示具有给定索引的项目。'
        return self.EnsureVisible(索引)

    @组件_异常检测
    def 取指定项目索引(self,查找的内容,区分大小写=False):
        '查找标签与给定字符串匹配的项目。返回项目索引'
        return self.FindString(查找的内容,区分大小写)

    def 取项目数(self):
        return self.GetCount()

    def 取可见项目数(self):
        '返回可以垂直放入列表框可见区域的项目数。'
        return self.GetCountPerPage()

    def 取选中项索引(self):
        return self.GetSelection()

    def 取选中范围索引(self):
        '返回一个列表包含所有选中项索引,用当前所选项目的位置填充一个整数数组。'
        return self.GetSelections()

    @组件_异常检测
    def 取指定项目文本(self, 索引):
        return self.GetString(索引)

    def 取首个可见项索引(self):
        '返回最顶部可见项目的索引。'
        return self.GetTopItem()

    @组件_异常检测
    def 取指定坐标索引(self,左边,顶边):
        '返回列表框内指定坐标处项目索引'
        return self.HitTest(左边,顶边)

    @组件_异常检测
    def 插入项目(self,插入位置,项目列表):
        return self.InsertItems(项目列表,插入位置)

    def 清空表项(self):
        self.Clear()

    @组件_异常检测
    def 置项目列表(self,项目列表):
        '会覆盖原有的项目列表'
        self.SetItems(项目列表)

    @组件_异常检测
    def 加入项目(self,项目):
        '支持单个或多个项目,多个项目使用列表传入，加入后会返回最后一个项目索引'
        return self.Append(项目)

    @组件_异常检测
    def 加入项目2(self,项目):
        '支持单个或多个项目,多个项目使用列表传入'
        self.AppendItems(项目)

    @组件_异常检测
    def 删除指定项目(self,索引):
        self.Delete(索引)

    @组件_异常检测
    def 项目是否选中(self,索引):
        return self.InsertItems(索引)

    def 表项是否按字母排序(self):
        return self.IsSorted()

    @组件_异常检测
    def 置顶指定项(self,索引):
        '将指定的项目设置为第一个可见项目。'
        self.SetFirstItem(索引)

    @组件_异常检测
    def 置指定项目背景色(self,索引,颜色):
        '在列表框中设置项目的背景色。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemBackgroundColour(索引,颜色)
        self.Refresh()

    @组件_异常检测
    def 置指定项目前景色(self,索引,颜色):
        '在列表框中设置项目的前景色。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemForegroundColour(索引,颜色)
        self.Refresh()

    @组件_异常检测
    def 置指定项目字体(self,索引,字体):
        '在列表框中设置项目的字体。仅在MSW上且wx.LB_OWNERDRAW设置了标志时有效。'
        self.SetItemFont(索引,字体)
        self.Refresh()

    @组件_异常检测
    def 置指定项目文本(self,索引,文本):
        self.SetString(索引,文本)

    @组件_异常检测
    def 置现行选中项_文本(self,项目文本):
        return self.SetStringSelection(项目文本)

    @组件_异常检测
    def 置现行选中项(self,索引):
        return self.SetSelection(索引)

    @组件_异常检测
    def 取选中项文本(self):
        return self.GetStringSelection()



import unittest

from .systemProcessingBase import *
from ubelt import *
from . import *


class TestSystemProcessingBase(unittest.TestCase):

    def test_1(self):
        pass
        data = 运行("ipconfig")
        print(data)

    def test_2(self):
        pass
        data = 取鼠标位置()
        print(data.x)
        print(data.y)



"""
xlwt 和 wlrd 封装的excel操作

可能需要

pip install xlutils
pip install xlwt
pip install xlrd

"""
import xlwt
import xlrd
from xlutils.copy import copy


class Excel():
    def __init__(self):
        pass

    def 打开Excel(self, 文件路径, 编辑=False, 保留样式=False):
        pass
        self.工作簿读操作 = xlrd.open_workbook(filename=文件路径,
                                         # logfile=sys.stdout,
                                         verbosity=0,
                                         # use_mmap=USE_MMAP,
                                         file_contents=None,
                                         encoding_override=None,
                                         formatting_info=保留样式,
                                         on_demand=False,
                                         ragged_rows=False)

        self.sheetw = None
        self.编辑 = 编辑
        if self.编辑:
            self.工作簿写操作 = self.复制为写操作excel()

    def 复制为写操作excel(self):
        wb = copy(self.工作簿读操作)
        wb.wb = wb
        return Excel写工作簿(wb)

    def 取sheet从索引(self, 索引):
        """
        索引从0开始
        """
        sh = self.工作簿读操作.sheet_by_index(索引)

        if self.编辑:
            self.sheetw = self.工作簿写操作.wb.get_sheet(索引)
        return ExcelSheet(sh, sheetw=self.sheetw)

    def 取sheet从名称(self, 名称):
        """
        取sheet名称()
        """
        sh = self.工作簿读操作.sheet_by_name(名称)

        if self.编辑:
            self.sheetw = self.工作簿写操作.wb.get_sheet(名称)

        return ExcelSheet(sheet=sh, sheetw=self.sheetw)

    def 取sheet数量(self):
        return self.工作簿读操作.nsheets

    def 取sheet名称(self):
        return self.工作簿读操作.sheet_names()

    def 保存(self, 文件路径):
        pass
        self.工作簿写操作.保存(文件路径)


class ExcelSheet():
    def __init__(self, sheet: xlrd.sheet.Sheet = None, sheetw: xlwt.Worksheet = None):
        pass
        self.sheet = sheet
        self.sheetw = sheetw

    def 取对象(self):
        return self.sheet

    def 取行数(self):
        return self.sheet.nrows

    def 取列数(self):
        return self.sheet.ncols

    def 取名称(self):
        return self.sheet.name

    def 取行所有内容(self, 行):
        return self.sheet.row_values(行)

    def 取列所有内容(self, 列):
        return self.sheet.col_values(列)

    def 取内容(self, 行, 列):
        return self.sheet.cell_value(行, 列)

    ### 要写出的库支持
    def 置内容(self, 行, 列, 内容, 样式=xlwt.Style.default_style):
        self.sheetw.write(行, 列, 内容, 样式)

    def 置列宽(self, 列, 宽度):
        self.sheetw.col(列).width = 宽度 * 256

    def 置行高(self, 行, 高度):
        self.sheetw.row(行).height_mismatch = True
        self.sheetw.row(行).height = 高度 * 20

    def 置图片从文件(self, row, col, filename, x=0, y=0, scale_x=1, scale_y=1):
        self.sheetw.insert_bitmap(row=row,
                                  col=col,
                                  filename=filename,
                                  x=x,
                                  y=y,
                                  scale_x=scale_x,
                                  scale_y=scale_y)

    def 置图片从数据(self, row, col, data, x=0, y=0, scale_x=1, scale_y=1):
        self.sheetw.insert_bitmap_data(row=row,
                                       col=col,
                                       data=data,
                                       x=x,
                                       y=y,
                                       scale_x=scale_x,
                                       scale_y=scale_y)


class Excel写工作簿():
    def __init__(self, wb: xlwt.Workbook = xlwt.Workbook()):
        self.wb = wb

    def 添加sheet(self, 名称):
        pass
        sh = self.wb.add_sheet(名称)
        return ExcelSheet(sheetw=sh)

    def 取sheet从名称(self, 名称):
        pass
        sh = self.wb.get_sheet(名称)
        return ExcelSheet(sheetw=sh)

    def 取sheet从索引(self, 索引):
        pass
        sh = self.wb.get_sheet(索引)
        return ExcelSheet(sheetw=sh)

    def 保存(self, 文件路径):
        pass
        self.wb.save(文件路径)


class Excel字体(xlwt.Font):
    # self.height = 0x00C8  # 200: this is font with height 10 points
    # self.italic = False
    # self.struck_out = False
    # self.outline = False
    # self.shadow = False
    # self.colour_index = 0x7FFF
    # self.bold = False
    # self._weight = 0x0190  # 0x02BC gives bold font
    # self.escapement = self.ESCAPEMENT_NONE
    # self.underline = self.UNDERLINE_NONE
    # self.family = self.FAMILY_NONE
    # self.charset = self.CHARSET_SYS_DEFAULT
    # self.name = 'Arial'

    @property
    def 轮廓(self):
        return self.outline

    @轮廓.setter
    def 轮廓(self, value):
        self.outline = value

    @property
    def 阴影(self):
        return self.shadow

    @阴影.setter
    def 阴影(self, value):
        self.shadow = value

    @property
    def 字体名称(self):
        return self.name

    @字体名称.setter
    def 字体名称(self, value):
        self.name = value

    @property
    def 颜色(self):
        return self.colour_index

    @颜色.setter
    def 颜色(self, value):
        self.colour_index = value

    @property
    def 大小(self):
        return self.height

    @大小.setter
    def 大小(self, value):
        self.height = 20 * value

    @property
    def 下划线(self):
        return self.underline

    @下划线.setter
    def 下划线(self, value):
        self.underline = value

    @property
    def 斜体(self):
        return self.italic

    @斜体.setter
    def 斜体(self, value):
        self.italic = value

    @property
    def 粗体(self):
        return self.bold

    @粗体.setter
    def 粗体(self, value):
        self.bold = value

    @property
    def 删除线(self):
        return self.struck_out

    @删除线.setter
    def 删除线(self, value):
        self.struck_out = value


class Excel对齐方式(xlwt.Alignment):
    # HORZ_GENERAL                = 0x00
    水平_常规 = 0x00
    # HORZ_LEFT                   = 0x01
    水平_左对齐 = 0x01
    # HORZ_CENTER                 = 0x02
    水平_居中对齐 = 0x02
    # HORZ_RIGHT                  = 0x03
    水平_右对齐 = 0x03
    # HORZ_FILLED                 = 0x04
    水平_填满 = 0x04
    # HORZ_JUSTIFIED              = 0x05 # BIFF4-BIFF8X
    水平_两端对齐 = 0x05  # BIFF4-BIFF8X
    # HORZ_CENTER_ACROSS_SEL      = 0x06 # Centred across selection (BIFF4-BIFF8X)
    水平_中心选择 = 0x06  # Centred across selection (BIFF4-BIFF8X)
    # HORZ_DISTRIBUTED            = 0x07 # Distributed (BIFF8X)
    水平_分散 = 0x07  # Distributed (BIFF8X)

    # VERT_TOP                    = 0x00
    垂直_顶端对齐 = 0x00
    # VERT_CENTER                 = 0x01
    垂直_居中对齐 = 0x01
    # VERT_BOTTOM                 = 0x02
    垂直_底端对齐 = 0x02
    # VERT_JUSTIFIED              = 0x03 # Justified (BIFF5-BIFF8X)
    垂直_两端对齐 = 0x03  # Justified (BIFF5-BIFF8X)
    # VERT_DISTRIBUTED            = 0x04 # Distributed (BIFF8X)
    垂直_分散 = 0x04  # Distributed (BIFF8X)

    @property
    def 水平方向(self):
        return self.horz

    @水平方向.setter
    def 水平方向(self, value):
        self.horz = value

    @property
    def 垂直方向(self):
        return self.vert

    @垂直方向.setter
    def 垂直方向(self, value):
        self.vert = value

    @property
    def 自动换行(self):
        return self.wrap

    @自动换行.setter
    def 自动换行(self, value):
        if value:
            self.wrap = 1
        else:
            self.wrap = 0


class Excel样式(xlwt.XFStyle):
    pass

    @property
    def 字体(self):
        return self.font

    @字体.setter
    def 字体(self, value):
        self.font = value

    pass

    @property
    def 对齐方式(self):
        return self.alignment

    @对齐方式.setter
    def 对齐方式(self, value):
        self.alignment = value

    @property
    def 边框(self):
        return self.borders

    @边框.setter
    def 边框(self, value):
        self.borders = value

    @property
    def 背景颜色(self):
        return self.pattern

    @背景颜色.setter
    def 背景颜色(self, value):
        self.pattern = value


def 打开Excel(文件路径, 编辑=False, 保留样式=False):
    """ 打开一个xls文件"""
    excel = Excel()
    excel.打开Excel(文件路径, 编辑=编辑, 保留样式=保留样式)

    return excel


def 创建Excel空工作簿():
    """ 创建一个空白的工作簿"""
    excel = Excel写工作簿()
    return excel


def 创建Excel字体(
        字体名称="宋体",
        轮廓=False,
        阴影=False,
        颜色=0,
        大小=12,
        下划线=False,
        斜体=False,
        粗体=False,
        删除线=False,
):
    字体 = Excel字体()
    字体.轮廓 = 轮廓
    字体.阴影 = 阴影
    字体.字体名称 = 字体名称
    字体.颜色 = 颜色
    字体.大小 = 大小
    字体.下划线 = 下划线
    字体.斜体 = 斜体
    字体.粗体 = 粗体
    字体.删除线 = 删除线
    return 字体


def 创建Excel对齐方式(
        水平方向=Excel对齐方式.水平_居中对齐,
        垂直方向=Excel对齐方式.垂直_居中对齐,
        自动换行=False
):
    对齐方式 = Excel对齐方式()
    对齐方式.水平方向 = 水平方向
    对齐方式.垂直方向 = 垂直方向
    对齐方式.自动换行 = 自动换行
    return 对齐方式


def 创建Excel边框(左边边框=0,
              右边边框=0,
              顶边边框=0,
              底边边框=0,
              左边颜色=0x40,
              右边颜色=0x40,
              顶边颜色=0x40,
              底边颜色=0x40,
              ):
    """
    # 细实线:1，小粗实线:2，细虚线:3，中细虚线:4，大粗实线:5，双线:6，细点虚线:7
    # 大粗虚线:8，细点划线:9，粗点划线:10，细双点划线:11，粗双点划线:12，斜点划线:13
    """
    borders = xlwt.Borders()
    borders.left = 左边边框
    borders.right = 右边边框
    borders.top = 顶边边框
    borders.bottom = 底边边框
    borders.left_colour = 左边颜色
    borders.right_colour = 右边颜色
    borders.top_colour = 顶边颜色
    borders.bottom_colour = 底边颜色
    return borders


def 创建Excel背景颜色(颜色索引=None):
    """
    0=黑色，1=白色，2=红色，3=绿色，4=蓝色，5=黄色，6=品红，7=青色，16=褐红色，17=深绿色，18=深蓝色，19=深黄色，几乎棕色），20=深品红，21=青色，22=浅灰色，23=深灰色
    """
    pattern = xlwt.Pattern()
    if 颜色索引 != None:
        pattern.pattern = xlwt.Pattern.SOLID_PATTERN
        pattern.pattern_fore_colour = 颜色索引

    return pattern


def 创建Excel保护(单元格锁定=1, 公式隐藏=0):
    protection = xlwt.Protection()
    protection.cell_locked = 单元格锁定
    protection.formula_hidden = 公式隐藏

    return protection


def 创建Excel样式(字体=创建Excel字体(), 对齐方式=创建Excel对齐方式(), 边框=创建Excel边框(), 背景颜色=创建Excel背景颜色(), 保护=创建Excel保护()):
    样式 = Excel样式()
    样式.字体 = 字体
    样式.对齐方式 = 对齐方式
    样式.边框 = 边框
    样式.背景颜色 = 背景颜色
    样式.protection = 保护

    # self.num_format_str = 'General'
    # self.font = Formatting.Font()
    # self.alignment = Formatting.Alignment()
    # self.borders = Formatting.Borders()
    # self.pattern = Formatting.Pattern()
    # self.protection = Formatting.Protection()
    return 样式



import wx.adv
import wx
from .wxControl import *


class 超级列表框(wx.ListCtrl, 公用方法):
    pass

    @组件_异常检测
    def 加入行(self,内容列表):
        '在末尾加入新一行数据，返回加入的行索引'
        return self.Append(内容列表)

    @组件_异常检测
    def 加入列(self,标题,对齐方式=2,宽度=-1):
        """
        在末尾加入新一列
        :param 标题: 自己定
        :param 对齐方式: 0.左对齐  1.右对齐 2.居中
        :param 宽度: 自己定
        :return:加入的列索引
        """
        return self.AppendColumn(标题,对齐方式,宽度)

    @组件_异常检测
    def 排列项目(self,排列方式=0):
        """
        在图标或小图标视图中排列项目
        :param 排列方式: 0.默认对齐方式  1.与控件的左侧对齐 2.与控件的顶部对齐 3.对齐网格
        :return:
        """
        return self.Arrange(排列方式)

    @组件_异常检测
    def 置选中状态(self,索引,选中=True):
        '使用复选框选中或取消选中 控件中的wx.ListItem'
        return self.CheckItem(索引,选中)

    def 全部删除(self):
        '删除所有项目和所有列'
        return self.ClearAll()

    def 删除所有列(self):
        return self.DeleteAllColumns()

    def 删除所有行(self):
        return self.DeleteAllItems()

    @组件_异常检测
    def 删除指定列(self,列索引):
        return self.DeleteColumn(列索引)

    @组件_异常检测
    def 删除指定行(self,行索引):
        return self.DeleteItem(行索引)

    @组件_异常检测
    def 开始编辑(self,行索引):
        '需要设置了 wx.LC_EDIT_LABELS 样式才能使用,开始编辑指定行的第一列'
        return self.EditLabel(行索引)

    @组件_异常检测
    def 启用或禁用交替行背景色(self,启用=True):
        '启用交替的行背景色（也称为斑马条纹）,该方法只能在虚拟报表模式（即具有LC_REPORT 和LC_VIRTUAL 样式）中为控件调用。'
        return self.EnableAlternateRowColours(启用)

    @组件_异常检测
    def 启用或禁用按键搜索(self,启用):
        '只匹配第一列，从键盘搜索项目时，如果当前输入的文本不匹配，则启用或禁用蜂鸣声。'
        return self.EnableBellOnNoMatch(启用)

    @组件_异常检测
    def 启用或禁用选择框(self,启用=True):
        '启用或禁用列表项的复选框'
        return self.EnableCheckBoxes(启用)

    @组件_异常检测
    def 启用或禁用系统主题样式(self,启用=True):
        return self.EnableSystemTheme(启用)

    @组件_异常检测
    def 保证显示(self,行索引):
        return self.EnsureVisible(行索引)

    @组件_异常检测
    def 保证显示2(self,行索引):
        return self.Focus(行索引)

    @组件_异常检测
    def 查找表项(self,查找的内容,开始索引=-1,模糊查找=False):
        '只查找第一列'
        return self.FindItem(开始索引,查找的内容,模糊查找)

    @组件_异常检测
    def 取交替行背景色(self):
        return self.GetAlternateRowColour()

    @组件_异常检测
    def 取列对象(self,列索引):
        return self.GetColumn(列索引)

    @组件_异常检测
    def 取列标题(self,列索引):
        return self.GetColumn(列索引).GetText()

    @组件_异常检测
    def 取列对齐方式(self,列索引):
        "对齐方式：0.左对齐 1.右对齐 2.居中"
        return self.GetColumn(列索引).GetAlign()

    @组件_异常检测
    def 取列宽(self,列索引):
        return self.GetColumnWidth(列索引)

    @组件_异常检测
    def 取列宽2(self,列索引):
        return self.GetColumn(列索引).GetWidth()

    def 取列数(self):
        return self.GetColumnCount()

    @组件_异常检测
    def 取可视列索引(self,索引):
        '一般用不上，除非你调整了列排序啥的我也没用过'
        return self.GetColumnOrder(索引)

    def 取列顺序索引(self):
        '一般用不上除非你改动了列，返回一个列表，包含列索引数值'
        return self.GetColumnsOrder()

    def 取可见行数(self):
        '返回列表框完全可见的行数'
        return self.GetCountPerPage()

    def 取编辑控件对象(self):
        '返回当前用于编辑标签的编辑控件。None如果没有标签被编辑，则返回'
        return self.GetEditControl()

    def 取现行选中项(self):
        '返回第一个选定的项目；如果未选择任何项目，则返回-1。'
        return self.GetFirstSelected()

    def 取现行焦点项(self):
        '获取当前焦点的项目；如果没有焦点，则返回-1。'
        return self.GetFocusedItem()

    @组件_异常检测
    def 取图片组列表(self,类型):
        '图片组类型：0.普通(大图列表)  1.小图列表  2.自定义列表  返回类型wx.ImageList'
        return self.GetImageList(类型)

    def 取行数(self):
        return self.GetItemCount()

    @组件_异常检测
    def 取行对象(self,行索引):
        return self.GetItem(行索引)

    @组件_异常检测
    def 取行背景色(self,行索引):
        return self.GetItemBackgroundColour(行索引)

    @组件_异常检测
    def 取行背景色2(self,行索引):
        return self.GetItem(行索引).GetBackgroundColour()

    @组件_异常检测
    def 取行字体(self,行索引):
        return self.GetItemFont(行索引)

    @组件_异常检测
    def 取行坐标(self,行索引):
        '返回指定行所在的x,y坐标'
        return self.GetItemPosition(行索引)

    @组件_异常检测
    def 取行矩形(self,行索引):
        '返回指定行的矩形'
        return self.GetItemRect(行索引)

    def 取图标间距(self):
        return self.GetItemSpacing()

    @组件_异常检测
    def 取行状态(self,行索引,类型=wx.LIST_STATE_SELECTED):
        '默认返回是否为现行选中项'
        return bool(self.GetItemState(行索引,类型))

    @组件_异常检测
    def 取标题(self,行索引,列索引):
        return self.GetItemText(行索引,列索引)

    @组件_异常检测
    def 取行文本颜色(self,行索引):
        '如果项目没有特定的颜色，则返回无效的颜色（而不是控件本身的默认前景色控件，因为这不允许区分与当前控件前景色相同颜色的项目和默认颜色的项目，因此，与控件始终具有相同的颜色）。'
        return self.GetItemTextColour(行索引)

    @组件_异常检测
    def 取下一选中项(self,当前索引):
        '返回指定行下面的现行选中项 没有就返回-1'
        return self.GetNextSelected(当前索引)

    def 取选中表项数(self):
        return self.GetSelectedItemCount()

    def 取文本颜色(self):
        return self.GetTextColour()

    def 取首个可见索引(self):
        '在列表或报表视图中获取最顶部可见项目的索引。'
        return self.GetTopItem()

    def 取最大尺寸(self):
        '''
        请注意，此功能仅在图标视图和小图标视图中有效，而在列表视图或报表视图中则无效（这是本机Win32控件的限制）。
        返回控件中所有项目采用的矩形。
        换句话说，如果控件客户端的大小等于此矩形的大小，则不需要滚动条，也不会留下可用空间。
        '''
        return self.GetViewRect()

    def 是否启用选择框(self):
        return self.HasCheckBoxes()

    def 是否带LC_REPORT样式(self):
        '单列或多列报表视图，带有可选标题。'
        return self.InReportView()

    @组件_异常检测
    def 插入列(self,插入位置,标题,对齐方式=2,宽度=-1):
        """
        在末尾加入新一列
        :param 标题: 自己定
        :param 对齐方式: 0.左对齐  1.右对齐 2.居中
        :param 宽度: 自己定
        :return:加入的列索引
        """
        return self.InsertColumn(插入位置,标题,对齐方式,宽度)

    @组件_异常检测
    def 插入行(self,插入位置,标题):
        '只能插入第一个标题'
        return self.InsertItem(插入位置,标题)

    @组件_异常检测
    def 插入图片(self,插入位置,图片索引):
        '与此控件和视图样式关联的图像列表的索引'
        return self.InsertItem(插入位置,图片索引)

    @组件_异常检测
    def 插入图文(self,插入位置,标题,图片索引):
        '插入图像/字符串项目。'
        return self.InsertItem(插入位置, 标题,图片索引)

    def 是否无表项(self):
        '具有某些列的控件如果没有行，则仍被认为是空的。'
        return self.IsEmpty()

    @组件_异常检测
    def 是否选中(self,行索引):
        '判断该行选择框是否勾选'
        return self.IsItemChecked(行索引)

    @组件_异常检测
    def 是否为选择项(self,索引):
        '返回True是否选择了该项目,不是选中'
        return self.IsSelected(索引)

    def 是否为虚拟报表(self):
        '返回True该控件当前是否在虚拟报表视图中。'
        return self.IsVirtual()

    @组件_异常检测
    def 表项是否可见(self,行索引):
        '检查项目是否可见。'
        return self.IsVisible(行索引)

    @组件_异常检测
    def 取指定行图片索引(self,行索引):
        '它应返回控件图像列表中项目图像的索引，如果没有图像，则返回-1'
        return self.OnGetItemImage(行索引)

    @组件_异常检测
    def 是否选中2(self,行索引):
        '它应该返回是否选中了指定 item 复选框。对于具有 使用复选框的样式的控件，必须在派生类中重写 此函数LC_VIRTUAL。'
        return self.OnGetItemIsChecked(行索引)

    @组件_异常检测
    def 取标题2(self,行索引,列索引):
        '它应返回包含 指定 的给定列文本的字符串item。对于具有 样式的控件，必须在派生类中重写 此函数LC_VIRTUAL。'
        return self.OnGetItemText(行索引,列索引)

    @组件_异常检测
    def 重画指定项目(self,行索引):
        return self.RefreshItem(行索引)

    @组件_异常检测
    def 重画指定范围项目(self,起始行,结束行):
        '正如RefreshItem 这仅对虚拟列表控件有用。起始项必须小于或等于结束项。重绘itemFrom 和itemTo之间的项目。'
        return self.RefreshItems(起始行,结束行)

    @组件_异常检测
    def 滚动滚动条(self,dx,dy):
        '如果处于图标，小图标或报告查看模式，则dx 指定要滚动的像素数。如果处于列表视图模式，则dx 指定要滚动的列数。dy 始终指定要垂直滚动的像素数。'
        return self.ScrollList(dx,dy)

    @组件_异常检测
    def 选择某项(self,行索引):
        '选择不是选中'
        return self.Select(行索引)

    @组件_异常检测
    def 置备用行背景色(self,颜色):
        '将备用行背景色设置为特定颜色。与一样EnableAlternateRowColours，此方法只能与具有LC_REPORT 和LC_VIRTUAL 样式的控件一起使用'
        return self.SetAlternateRowColour(颜色)

    @组件_异常检测
    def 置列标题(self,列索引,标题):
        Item = self.GetColumn(列索引)
        Item.SetText(标题)
        return self.SetColumn(列索引,Item)

    @组件_异常检测
    def 置列宽(self,列索引,宽度):
        return self.SetColumnWidth(列索引,宽度)

    @组件_异常检测
    def 置列宽2(self,列索引,宽度):
        Item = self.GetColumn(列索引)
        Item.SetWidth(宽度)
        return self.SetColumn(列索引,Item)

    @组件_异常检测
    def 置列对齐方式(self,列索引,对齐方式):
        '对齐方式: 0.左对齐  1.右对齐 2.居中'
        Item = self.GetColumn(列索引)
        Item.SetAlign(对齐方式)
        return self.SetColumn(列索引,Item)

    @组件_异常检测
    def 置列图片(self,列索引,图片):
        return self.SetColumnImage(列索引,图片)

    @组件_异常检测
    def 置列排序位置(self,排序列表):
        '修改列的位置'
        return self.SetColumnsOrder(排序列表)

    @组件_异常检测
    def 置列标题字体颜色(self,attr):
        '更改用于列表控件标题的字体和颜色。'
        return self.SetHeaderAttr(attr)

    @组件_异常检测
    def 置关联图片列表(self,图片列表):
        return self.SetImageList(图片列表)

    @组件_异常检测
    def 置标题(self,行索引,列索引,标题):
        return self.SetItem(行索引,列索引,标题)

    @组件_异常检测
    def 置图文(self,行索引,列索引,标题,图片索引):
        return self.SetItem(行索引,列索引,标题,图片索引)

    @组件_异常检测
    def 置行色(self,行索引,颜色):
        return self.SetItemBackgroundColour(行索引,颜色)

    @组件_异常检测
    def 置图片(self,行索引,列索引,图片索引):
        return self.SetItemColumnImage(行索引,列索引,图片索引)

    @组件_异常检测
    def 置行数(self,行数):
        '此方法只能与虚拟列表控件一起使用。 没用过'
        return self.SetItemCount(行数)

    @组件_异常检测
    def 置行字体(self,行索引,字体):
        return self.SetItemFont(行索引,字体)

    @组件_异常检测
    def 置选中状态图片(self,行索引,选中图索引,未选中图索引):
        '设置与项目关联的未选择和选择的图像。'
        return self.SetItemImage(行索引,选中图索引,未选中图索引)

    @组件_异常检测
    def 置项目坐标(self,项目索引,x,y):
        '在图标或小图标视图中设置项目的位置。'
        return self.SetItemPosition(项目索引,(x,y))

    @组件_异常检测
    def 置选择状态(self,行索引,是否选择):
        状态 = 4 if 是否选择 else 0
        return self.SetItemState(行索引, 状态, wx.LIST_STATE_SELECTED)

    @组件_异常检测
    def 置标题_首列(self,行索引,标题):
        return self.SetItemText(行索引,标题)

    @组件_异常检测
    def 置行文本颜色(self,行索引,颜色):
        return self.SetItemTextColour(行索引,颜色)

    @组件_异常检测
    def 添加列表样式(self,样式):
        return self.SetSingleStyle(样式,True)

    @组件_异常检测
    def 删除列表样式(self,样式):
        return self.SetSingleStyle(样式,False)

    @组件_异常检测
    def 置全部文本颜色(self,颜色):
        return self.SetTextColour(颜色)

    @组件_异常检测
    def 置窗口新样式(self,样式):
        return self.SetWindowStyleFlag(样式)



import wx
from .wxControl import *


class 图片框(wx.StaticBitmap, 公用方法):
    pass

    @property
    def 图片(self):
        return self.取图片()

    @图片.setter
    def 图片(self, value):
        return self.置图片(value)

    @property
    def 图标(self):
        return self.取图标()

    @图标.setter
    def 图标(self, value):
        return self.置图标(value)


    def 取图片(self):
        '返回控件中当前使用的位图。'
        return self.GetBitmap()

    def 取图标(self):
        '返回控件中当前使用的图标。'
        return self.GetIcon()

    def 取缩放模式(self):
        '模式： 0.以原始大小显示 1.比例填充  2.通过保持纵横比，缩放位图以适合控件的大小。  3.缩放位图以填充控件的大小。'
        return self.GetScaleMode()

    @组件_异常检测
    def 置图片(self, 图片):
        return self.SetBitmap(图片)

    @组件_异常检测
    def 置图标(self, 图标):
        return self.SetIcon(图标)

    @组件_异常检测
    def 置缩放模式(self, 模式):
        '模式： 0.以原始大小显示 1.比例填充  2.通过保持纵横比，缩放位图以适合控件的大小。  3.缩放位图以填充控件的大小。'
        return self.SetScaleMode(模式)



"""
一些ui界面的函数

"""
import datetime
import traceback

def 组件_异常检测(function):
    '装饰器'
    def box(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except:
            print(function.__name__, "函数发生异常")
            print("错误发生时间：", str(datetime.datetime.now()))
            print("错误的详细情况：", traceback.format_exc())
    return box

@组件_异常检测
def 窗口_取窗口句柄(组件):
    '取wxpython组件的窗口句柄'
    return 组件.GetHandle()

@组件_异常检测
def 窗口_取组件祖组件(组件):
    '取wxpython组件的上上层组件,[当前-父组件-祖组件]'
    return 组件.GetGrandParent()

@组件_异常检测
def 窗口_对齐(组件,方向=12):
    '默认居中,使组件在父组件内对齐,主窗口则在屏幕中间,1.左上 4/5.顶边 8/9.左边 12/13.居中'
    return 组件.Center(方向)

@组件_异常检测
def 窗口_取桌面相对坐标(组件,x=0,y=0):
    '返回相对于此组件的坐标转换为屏幕坐标,x,y为偏移位置,0为当前'
    return 组件.ClientToScreen(x,y)

@组件_异常检测
def 窗口_关闭(窗口,关闭=True):
    '用来关闭窗口'
    return 窗口.Close(关闭)


@组件_异常检测
def 窗口_销毁(窗口):
    '备注写的这个方法不会立即销毁窗口,会等事件执行后才安全的销毁'
    return 窗口.Destroy()

@组件_异常检测
def 窗口_销毁所有子窗口(组件):
    '销毁窗口下所有的子窗口,组件'
    return 组件.DestroyChildren()

@组件_异常检测
def 窗口_销毁2(窗口):
    '官方解释：计划在不久的将来销毁该窗口,每当销毁可能发生得太早时（例如，当该窗口或其子级仍在事件队列中等待时），都应使用此方法'
    return 窗口.DestroyLater()

@组件_异常检测
def 窗口_禁用(组件):
    '组件禁用后连同子级组件也无法点击移动'
    return 组件.Disable()

@组件_异常检测
def 窗口_禁用2(组件):
    '启用或禁用用于用户输入的窗口'
    return 组件.Enable(False)

@组件_异常检测
def 窗口_允许拖放文件(组件,允许=True):
    '允许接收拖放文件'
    return 组件.DragAcceptFiles(允许)

@组件_异常检测
def 窗口_ID匹配组件(父窗口,id):
    '在父窗口下查找返回该ID的组件'
    return 父窗口.FindWindow(id)

@组件_异常检测
def 窗口_ID匹配组件2(父窗口,id):
    '在父窗口下查找返回匹配到的第一个该ID的组件,可使用wx.FindWindowById全程序查找'
    return 父窗口.FindWindowById(id)

@组件_异常检测
def 窗口_取键盘焦点组件(父窗口):
    '在父窗口下查找当前具有键盘焦点的窗口或控件'
    return 父窗口.FindFocus()

@组件_异常检测
def 窗口_标题匹配组件(父窗口,标题):
    '通过组件标题查找返回匹配到的第一个组件,可使用wx.FindWindowByLabel全程序查找'
    return 父窗口.FindWindowByLabel(标题)

@组件_异常检测
def 窗口_名称匹配组件(父窗口,组件名):
    '通过组件标题查找返回匹配到的第一个组件,可使用wx.FindWindowByName全程序查找'
    return 父窗口.FindWindowByName(组件名)

@组件_异常检测
def 窗口_自动调整尺寸(组件):
    '调整窗口大小以适合其最佳大小。'
    return 组件.Fit()

@组件_异常检测
def 窗口_自动调整内部尺寸(组件):
    '与相似Fit，但是调整窗户的内部（虚拟）尺寸,主要用于滚动窗口，以在调整大小而不会触发大小事件的情况下重置滚动条，和/或不带内部大小调整器的滚动窗口。如果没有子窗口，此功能同样不会执行任何操作。'
    return 组件.FitInside()

@组件_异常检测
def 窗口_禁止重画(组件):
    '冻结窗口，换句话说，阻止任何更新在屏幕上发生，窗口根本不会重绘'
    return 组件.Freeze()

@组件_异常检测
def 窗口_允许重画(组件):
    '重新启用窗口更新'
    return 组件.Thaw()

@组件_异常检测
def 窗口_取背景颜色(组件):
    '返回窗口的背景色,格式:(240, 240, 240, 255)'
    return 组件.GetBackgroundColour()

@组件_异常检测
def 窗口_取样式(组件):
    '样式：0.默认背景样式值,1.使用由系统或当前主题确定的默认背景,2.指示仅在用户定义的EVT_PAINT处理程序中擦除背景,3.无介绍,4.表示未擦除窗口背景，从而使父窗口得以显示'
    return 组件.GetBackgroundStyle()

@组件_异常检测
def 窗口_取最小可接受尺寸(组件):
    '回窗口的最佳可接受最小尺寸,返回格式:(宽度,高度)，高度不包含标题栏高度'
    return 组件.GetBestSize()

@组件_异常检测
def 窗口_取最大可接受尺寸(组件):
    '回窗口的最佳可接受最小尺寸,返回格式:(宽度,高度)，高度不包含标题栏高度'
    return 组件.GetBestVirtualSize()

@组件_异常检测
def 窗口_取边框样式(组件):
    '获取此窗口的标志的边框'
    return 组件.GetBorder()

@组件_异常检测
def 窗口_取额外样式(组件):
    '窗口的额外样式位'
    return 组件.GetExtraStyle()

@组件_异常检测
def 窗口_取字体高度(组件):
    '返回此窗口的字符高度'
    return 组件.GetCharHeight()

@组件_异常检测
def 窗口_取平均字符宽度(组件):
    '返回此窗口的平均字符宽度'
    return 组件.GetCharWidth()

@组件_异常检测
def 窗口_遍历下级组件(组件):
    '遍历组件下的子级组件,返回在WindowList 列表里'
    return 组件.GetChildren()

@组件_异常检测
def 窗口_取字体及颜色(组件):
    '返回字体,背景颜色,前景颜色，#(<wx._core.Font object at 0x000002140997DB88>, wx.Colour(240, 240, 240, 255), wx.Colour(0, 0, 0, 255))'
    结果 = 组件.GetClassDefaultAttributes()
    return 结果.font,结果.colBg,结果.colFg

@组件_异常检测
def 窗口_取矩形(组件):
    '返回窗口矩形：(左边,顶边,宽度,高度)'
    return 组件.GetRect()

@组件_异常检测
def 窗口_取矩形2(组件):
    '返回窗口矩形：(0,0,宽度,高度)'
    return 组件.GetClientRect()

@组件_异常检测
def 窗口_取宽高(组件):
    '返回窗口实际宽高：(宽度,高度)'
    return 组件.GetClientSize()

@组件_异常检测
def 窗口_取宽高2(组件):
    '将窗口的最佳大小合并为最小大小，然后返回结果,返回宽高：(宽度,高度)'
    return 组件.GetEffectiveMinSize()

@组件_异常检测
def 窗口_取字体(组件):
    '返回此窗口的字体'
    return 组件.GetFont()

@组件_异常检测
def 窗口_取前景色(组件):
    '返回窗口的前景色'
    return 组件.GetForegroundColour()

@组件_异常检测
def 窗口_取标记ID(组件):
    '返回窗口的标识符'
    return 组件.GetId()

@组件_异常检测
def 窗口_取标题(组件):
    '返回窗口的标题'
    return 组件.GetLabel()

@组件_异常检测
def 窗口_置工作区宽高(组件,宽度,高度):
    '设置组件工作区的宽高(不包含边框,标题栏的宽高)'
    return 组件.SetClientSize(宽度,高度)

@组件_异常检测
def 窗口_取工作区最小宽高(组件):
    '返回窗口的工作区的最小大小，这向sizer布局机制指示这是其工作区的最小所需大小'
    return 组件.GetMinClientSize()

@组件_异常检测
def 窗口_取最小宽高(组件):
    '返回窗口的最小大小，这向sizer布局机制指示这是最小所需大小'
    return 组件.GetMinSize()

@组件_异常检测
def 窗口_取组件名称(组件):
    '返回窗口的名称'
    return 组件.GetName()

@组件_异常检测
def 窗口_取下一窗口(组件):
    '返回此窗口之后的下一个窗口(同一级窗口里)'
    return 组件.GetNextSibling()

@组件_异常检测
def 窗口_取上一窗口(组件):
    '返回父级的子级中前一个的前一个窗口'
    return 组件.GetPrevSibling()

@组件_异常检测
def 窗口_取父级窗口(组件):
    '返回窗口的父级，或者返回没有父级的窗口None'
    return 组件.GetParent()


@组件_异常检测
def 窗口_弹出菜单(组件,菜单,左边,顶边):
    '此函数在此窗口中的给定位置显示一个弹出菜单，并返回所选的ID'
    return 组件.GetPopupMenuSelectionFromUser(菜单,左边,顶边)

@组件_异常检测
def 窗口_取左边顶边(组件):
    '这将获得相对于子窗口的父窗口或相对于顶级窗口的显示原点的窗口位置（以像素为单位）,格式：(左边,顶边)'
    return 组件.DirDialog()

@组件_异常检测
def 窗口_取窗口相对屏幕坐标(组件):
    '返回窗口在屏幕坐标中的位置，无论该窗口是子窗口还是顶级窗口,格式:(相对于屏幕的左边,相对于屏幕的顶边)'
    return 组件.GetScreenPosition()

@组件_异常检测
def 窗口_取窗口相对屏幕矩形(组件):
    '返回窗口在屏幕坐标中的位置，无论该窗口是子窗口还是顶级窗口,格式:(相对于屏幕的左边,相对于屏幕的顶边,组件宽度,组件高度)'
    return 组件.GetScreenRect()

@组件_异常检测
def 窗口_取内置滚动条位置(组件,方向):
    '返回内置滚动条的位置,方向：4.横向滚动条 8.纵向滚动条'
    return 组件.GetScrollPos(方向)

@组件_异常检测
def 窗口_取内置滚动条范围(组件,方向):
    '返回内置滚动条范围,方向：4.横向滚动条 8.纵向滚动条'
    return 组件.GetScrollRange(方向)

@组件_异常检测
def 窗口_取内置滚动条缩略图大小(组件,方向):
    '返回内置滚动条的缩略图大小,方向：4.横向滚动条 8.纵向滚动条'
    return 组件.GetScrollThumb(方向)

@组件_异常检测
def 窗口_置滚动条位置(组件,方向,位置,重绘):
    '设置滚动条位置,方向可选4或8,重绘True或False(设置内置滚动条之一的位置)'
    return 组件.SetScrollPos(方向,位置,重绘)

@组件_异常检测
def 窗口_置滚动条属性(组件,方向,位置,可见大小,最大位置,重绘):
    '''
    设置滚动条位置,方向可选4或8,重绘True或False(设置内置滚动条之一的位置)
    假设您希望使用相同的字体显示50行文本。窗口的大小设置为一次只能看到16行。您将使用：
    self.SetScrollbar(wx.VERTICAL, 0, 16, 50)
    '''
    return 组件.SetScrollbar(方向,位置,可见大小,最大位置,重绘)

@组件_异常检测
def 窗口_取完整窗口宽高(组件):
    '返回整个窗口的大小（以像素为单位），包括标题栏，边框，滚动条等。如果此窗口是顶级窗口，并且当前已最小化，则返回的大小是还原的窗口大小，而不是窗口图标的大小'
    return 组件.GetSize()

@组件_异常检测
def 窗口_是否使用系统主题设置背景(组件):
    '窗口是否使用系统主题绘制其背景'
    return 组件.GetThemeEnabled()

@组件_异常检测
def 窗口_取顶级窗口(组件):
    '返回此窗口（顶级窗口）的第一个祖先'
    return 组件.GetTopLevelParent()

@组件_异常检测
def 窗口_取虚拟宽高(组件):
    '出错返回False,这将获取窗口的虚拟大小,它返回窗口的客户端大小，但是在调用SetVirtualSize 它之后，将返回使用该方法设置的大小'
    return 组件.GetVirtualSize()

@组件_异常检测
def 窗口_是否有焦点(组件):
    '窗口（或在复合控件的情况下，其主子窗口）是否具有焦点'
    return 组件.HasFocus()

@组件_异常检测
def 窗口_是否有滚动条(组件,方向):
    '返回此窗口当前是否具有该方向的滚动条,方向：4.横向滚动条 8.纵向滚动条'
    return 组件.HasScrollbar(方向)

@组件_异常检测
def 窗口_是否透明(组件):
    '返回此窗口背景是否透明（例如，对于 wx.StaticText），并应显示父窗口背景'
    return 组件.HasTransparentBackground()

@组件_异常检测
def 窗口_隐藏(组件):
    '此功能可隐藏一个窗口'
    return 组件.Hide()

@组件_异常检测
def 窗口_隐藏带特效(组件,效果,效果时长):
    '''
    此功能可隐藏一个窗口并使用特殊的视觉效果
    效果：0.无效果，1.向左滚动窗口，2.向右滚动窗口，3.将窗口滚动到顶部，4.将窗口滚动到底部，5.向左滑动窗口，6.向右滑动窗口，7.将窗口滑动到顶部，8.将窗口滑动到底部，9.淡入或淡出效果，10.扩大或崩溃的作用
    效果时长：单位毫秒
    '''
    return 组件.HideWithEffect(效果,效果时长)

@组件_异常检测
def 窗口_显示带特效(组件,效果,效果时长):
    '''
    此功能可隐藏一个窗口并使用特殊的视觉效果
    效果：0.无效果，1.向左滚动窗口，2.向右滚动窗口，3.将窗口滚动到顶部，4.将窗口滚动到底部，5.向左滑动窗口，6.向右滑动窗口，7.将窗口滑动到顶部，8.将窗口滑动到底部，9.淡入或淡出效果，10.扩大或崩溃的作用
    效果时长：单位毫秒
    '''
    return 组件.ShowWithEffect(效果,效果时长)

@组件_异常检测
def 窗口_是否继承父级背景色(组件):
    '如果此窗口从其父级继承背景色，则返回True'
    return 组件.InheritsBackgroundColour()


@组件_异常检测
def 窗口_是否继承父级前景色(组件):
    '如果此窗口从其父级继承前景色，则返回True'
    return 组件.InheritsForegroundColour()

@组件_异常检测
def 窗口_重置缓存最佳大小(组件):
    '重置缓存的最佳大小值，以便下次需要时重新计算'
    return 组件.InvalidateBestSize()

@组件_异常检测
def 窗口_是否正在销毁(组件):
    '此窗口是否正在销毁中'
    return 组件.IsBeingDeleted()

@组件_异常检测
def 窗口_是否为下级窗口(组件,对比组件):
    '检查指定的窗口是否是该窗口的后代,窗口是否为该窗口的后代（例如，子代或孙代或子孙或……）返回True'
    return 组件.IsDescendant(对比组件)

@组件_异常检测
def 窗口_是否禁用(组件):
    '是否启用了窗口，即是否接受用户输入'
    return 组件.IsEnabled()

@组件_异常检测
def 窗口_是否可获取焦点(组件):
    '判断窗口是否可以获取焦点'
    return 组件.IsFocusable()

@组件_异常检测
def 窗口_是否禁止重画(组件):
    '判断窗口是否可已经禁止重画'
    return 组件.IsFrozen()

@组件_异常检测
def 窗口_是否始终显示滚动条(组件,方向):
    '判断滚动条是否始终显示,方向：4.横向滚动条 8.纵向滚动条'
    return 组件.IsScrollbarAlwaysShown(方向)

@组件_异常检测
def 窗口_是否隐藏(组件):
    '判断是否调用命令隐藏了窗口,最小化,遮挡,不算隐藏'
    return 组件.IsShown()

@组件_异常检测
def 窗口_是否显示在屏幕上(组件):
    '判断是否调用命令隐藏了窗口,最小化,遮挡,不算隐藏'
    return 组件.IsShownOnScreen()

@组件_异常检测
def 窗口_是否启用(组件):
    '是否从本质上启用了此窗口，False否则返回'
    return 组件.IsThisEnabled()

@组件_异常检测
def 窗口_是否为顶级窗口(组件):
    '窗口是否为顶级窗口'
    return 组件.IsTopLevel()

@组件_异常检测
def 窗口_向下滚动(组件):
    '与ScrollLines （1）相同，返回True是否滚动窗口，False如果窗口已经在底部，则什么也不做'
    return 组件.LineDown()

@组件_异常检测
def 窗口_向上滚动(组件):
    '与ScrollLines （-1）相同,返回True是否滚动窗口，False如果窗口已经在顶部，则什么也不做'
    return 组件.LineUp()

@组件_异常检测
def 窗口_滚动_页(组件,滚动页数=1):
    '滚动页数:向上滚动1次为-1,向下为1'
    return 组件.ScrollPages(滚动页数)

@组件_异常检测
def 窗口_滚动_行(组件,滚动行数=1):
    '滚动行数:向上滚动1次为-1,向下为1'
    return 组件.ScrollLines(滚动行数)

@组件_异常检测
def 窗口_移动左边顶边(组件,左边,顶边):
    '调整移动窗口的左边跟顶边位置'
    return 组件.Move(左边,顶边)

@组件_异常检测
def 窗口_移动左边顶边2(组件,左边,顶边):
    '调整移动窗口的左边跟顶边位置'
    return 组件.SetPosition((左边,顶边))

@组件_异常检测
def 窗口_移动(组件,左边=-1,顶边=-1,宽度=-1,高度=-1):
    '调整移动窗口的左边跟顶边位置并重新设置宽度跟高度,不想调整的填-1'
    return 组件.SetSize(左边,顶边,宽度,高度)

@组件_异常检测
def 窗口_设置切换顺序_上(组件,上一个组件):
    '调整TAB切换的顺序,当上一个组件按TAB后焦点就会到当前组件上'
    return 组件.MoveAfterInTabOrder(上一个组件)

@组件_异常检测
def 窗口_设置切换顺序_下(组件,下一个组件):
    '调整TAB切换的顺序,当前组件按TAB后焦点会切换到下一个组件'
    return 组件.MoveBeforeInTabOrder(下一个组件)

@组件_异常检测
def 窗口_生成组件ID(组件):
    '创建一个新的ID或当前未使用的ID范围,格式:-31987'
    return 组件.NewControlId()

@组件_异常检测
def 窗口_重绘指定区域(组件,矩形=(0,0,0,0),擦除背景=True):
    '重绘指定矩形的内容：仅对其内部的区域进行重绘'
    return 组件.RefreshRect(矩形,擦除背景)

@组件_异常检测
def 窗口_修改父级窗口(组件,新父级组件):
    '即该窗口将从其当前父窗口中移除加入到新的父级窗口下'
    return 组件.Reparent(新父级组件)

@组件_异常检测
def 窗口_桌面坐标转窗口内坐标(组件,x,y):
    '从屏幕转换为客户端窗口内工作区坐标,'
    return 组件.ScreenToClient(x,y)

@组件_异常检测
def 窗口_到最顶层(组件):
    return 组件.Raise()

@组件_异常检测
def 窗口_到最底层(组件):
    return 组件.Lower()

@组件_异常检测
def 窗口_是否已设置背景色(组件):
    return 组件.UseBackgroundColour()

@组件_异常检测
def 窗口_是否已设置前景色(组件):
    return 组件.UseForegroundColour()

@组件_异常检测
def 窗口_置背景颜色(组件,颜色):
    return 组件.SetBackgroundColour(颜色)

@组件_异常检测
def 窗口_单独置背景颜色(组件,颜色):
    '设置窗口的背景色，但防止其被该窗口的子级继承'
    return 组件.SetOwnBackgroundColour(颜色)

@组件_异常检测
def 窗口_置前景颜色(组件,颜色):
    return 组件.SetForegroundColour(颜色)

@组件_异常检测
def 窗口_单独置前景颜色(组件,颜色):
    '设置窗口的前景色，但防止其被该窗口的子代继承'
    return 组件.SetOwnForegroundColour(颜色)

@组件_异常检测
def 窗口_置标识ID(组件,ID):
    return 组件.SetId(ID)

@组件_异常检测
def 窗口_置宽高(组件,宽度,高度):
    return 组件.SetInitialSize((宽度,高度))

@组件_异常检测
def 窗口_置最大宽高(组件,宽度,高度):
    '设置整个窗口最大尺寸范围'
    return 组件.SetMaxSize((宽度,高度))

@组件_异常检测
def 窗口_置最小宽高(组件,宽度,高度):
    '设置整个窗口最大尺寸范围'
    return 组件.SetMinSize((宽度,高度))

@组件_异常检测
def 窗口_置工作区最大宽高(组件,宽度,高度):
    '设置窗口的最大客户端大小(不包含标题栏菜单栏状态栏的尺寸)，以向sizer布局机制指示这是其客户端区域的最大可能大小'
    return 组件.SetMaxClientSize((宽度,高度))

@组件_异常检测
def 窗口_置工作区最小宽高(组件,宽度,高度):
    '设置窗口的最大客户端大小(不包含标题栏菜单栏状态栏的尺寸)，以向sizer布局机制指示这是其客户端区域的最大可能大小'
    return 组件.SetMinClientSize((宽度,高度))

@组件_异常检测
def 窗口_置虚拟宽高(组件,宽度,高度):
    '设置窗口的虚拟大小（以像素为单位）'
    return 组件.SetVirtualSize((宽度,高度))

@组件_异常检测
def 窗口_置标题(组件,标题):
    return 组件.SetLabel(标题)

@组件_异常检测
def 窗口_置名称(组件,名称):
    return 组件.SetName(名称)

@组件_异常检测
def 窗口_是否允许透明(组件):
    return 组件.CanSetTransparent()


@组件_异常检测
def 窗口_置透明度(组件,透明度):
    '设置窗口与透明度,范围0-255(0.完全透明,255完全不透明)'
    return 组件.SetTransparent(透明度)

@组件_异常检测
def 窗口_置主题样式(组件,样式):
    '窗口样式:0.默认(可擦除背景),1.跟随系统主题,2.指示仅在用户定义的EVT_PAINT处理程序中擦除背景,3.表示未擦除窗口背景，从而使父窗口得以显示,4.无描述。'
    return 组件.SetBackgroundStyle(样式)

@组件_异常检测
def 窗口_置窗口样式(组件,样式):
    '样式:0.无边框,536870912.右上角无按钮,更多样式百度'
    return 组件.SetWindowStyleFlag(样式)

@组件_异常检测
def 窗口_刷新重绘(组件,删除背景=False):
    '导致GTK1重新绘制此窗口及其所有子级（除非未实现此子级）'
    return 组件.Refresh(删除背景)

@组件_异常检测
def 窗口_刷新重绘2(组件):
    '调用此方法将立即重新绘制窗口的无效区域及其所有子级的对象（通常仅在控制流返回事件循环时才发生）'
    return 组件.Update()

@组件_异常检测
def 窗口_显示或隐藏(组件,是否显示=True):
    '显示或隐藏窗口'
    return 组件.Show(是否显示)

@组件_异常检测
def 窗口_移动鼠标(组件,x,y):
    '将指针移动到窗口上的指定位置'
    return 组件.WarpPointer(x,y)

@组件_异常检测
def 窗口_置鼠标光标样式(组件,样式):
    '''样式:
    0:无描述
    1:标准箭头光标。
    2:指向右侧的标准箭头光标。
    3:靶心光标。
    4:矩形字符光标。
    5:十字光标。
    6:手形光标。
    7:工字梁光标（垂直线）。
    8:表示鼠标左键按下。
    9:放大镜图标。
    10:表示按下中间按钮的鼠标。
    11:不可输入的符号光标。
    12:画笔光标。
    13:铅笔光标。
    14:指向左的光标。
    15:指向右的光标。
    16:箭头和问号。
    17:表示按下了右键的鼠标。
    18:调整大小的光标指向NE-SW。
    19:调整大小的光标指向N-S。
    20:调整大小的光标指向NW-SE。
    21:调整大小的光标指向W-E。
    22:一般大小的游标。
    23:Spraycan游标。
    24:等待光标。
    25:监视光标。
    26:透明光标。
    27:带有标准箭头的等待光标。
    28:无描述。
    '''
    return 组件.SetCursor(wx.Cursor(样式))

@组件_异常检测
def 窗口_设置字体(组件,字体名,大小,粗细,下划线):
    '窗口样式:0.默认(可擦除背景),1.跟随系统主题,2.指示仅在用户定义的EVT_PAINT处理程序中擦除背景,3.表示未擦除窗口背景，从而使父窗口得以显示,4.无描述。'
    return 组件.SetFont(wx.Font(大小,wx.DEFAULT, wx.NORMAL, 粗细, 下划线, 字体名))





@组件_异常检测
def 程序_取指定坐标处组件(x,y):
    '传入桌面上坐标'
    return wx.FindWindowAtPoint((x,y))

@组件_异常检测
def 程序_取鼠标处组件跟坐标():
    '取当前鼠标下面的组件及坐标,返回格式:(组件,(x,y)),返回的坐标是相对于桌面的坐标'
    return wx.FindWindowAtPointer()

@组件_异常检测
def 程序_取屏幕工作区矩形():
    '取屏幕工作区矩形(不包含任务栏宽高),格式：(0,0,1920,1040) 任务栏占了40'
    return wx.GetClientDisplayRect()

@组件_异常检测
def 程序_取屏幕分辨率():
    '返回格式:(1920,1080)'
    return wx.GetDisplaySize()

@组件_异常检测
def 程序_取屏幕尺寸():
    '返回以毫米为单位的显示尺寸，格式:(508,286)'
    return wx.GetDisplaySizeMM()

@组件_异常检测
def 程序_恢复默认鼠标光标():
    '对于应用程序中的所有窗口，将光标更改回原始光标'
    return wx.EndBusyCursor()

@组件_异常检测
def 程序_重置所有鼠标光标(光标类型):
    '''将光标更改为应用程序中所有窗口的给定光标
    光标类型:
    0:无描述
    1:标准箭头光标。
    2:指向右侧的标准箭头光标。
    3:靶心光标。
    4:矩形字符光标。
    5:十字光标。
    6:手形光标。
    7:工字梁光标（垂直线）。
    8:表示鼠标左键按下。
    9:放大镜图标。
    10:表示按下中间按钮的鼠标。
    11:不可输入的符号光标。
    12:画笔光标。
    13:铅笔光标。
    14:指向左的光标。
    15:指向右的光标。
    16:箭头和问号。
    17:表示按下了右键的鼠标。
    18:调整大小的光标指向NE-SW。
    19:调整大小的光标指向N-S。
    20:调整大小的光标指向NW-SE。
    21:调整大小的光标指向W-E。
    22:一般大小的游标。
    23:Spraycan游标。
    24:等待光标。
    25:监视光标。
    26:透明光标。
    27:带有标准箭头的等待光标。
    28:无描述。
    '''
    return wx.BeginBusyCursor(wx.Cursor(光标类型))

@组件_异常检测
def 程序_关闭2():
    "立即结束程序"
    wx.Abort()

@组件_异常检测
def 程序_关闭():
    "立即结束程序,会卡顿下"
    wx.Exit()

@组件_异常检测
def 程序_系统错误代码转提示文本(code):
    "返回与给定系统错误代码对应的错误消息,示例：code=3 返回:系统找不到指定的路径。"
    return wx.SysErrorMsgStr(3)

@组件_异常检测
def 程序_电脑关机():
    "立即结束程序,会卡顿下"
    wx.Shutdown(2)

@组件_异常检测
def 程序_电脑重启():
    "立即结束程序,会卡顿下"
    wx.Shutdown(4)

@组件_异常检测
def 程序_延时_微秒(时间):
    "延时单位(微秒)1秒=1000000微秒"
    wx.MicroSleep(时间)

@组件_异常检测
def 程序_延时_毫秒(时间):
    "延时单位(毫秒)1秒=1000毫秒"
    wx.MilliSleep(时间)

@组件_异常检测
def 程序_延时_秒(时间):
    wx.Sleep(时间)

@组件_异常检测
def 程序_取本地英文时间():
    "返回时间示例：Sun Aug 16 15:57:41 2020"
    return wx.Now()

@组件_异常检测
def 程序_取程序对象():
    "返回当前应用程序对象"
    return wx.GetApp()

@组件_异常检测
def 程序_取程序顶级窗口列表():
    "返回应用程序顶级窗口的类似列表的对象(返回顶级窗口的对象列表)"
    return wx.GetTopLevelWindows()

@组件_异常检测
def 程序_取计算机名():
    "返回当前应用程序对象"
    return wx.GetHostName()

@组件_异常检测
def 程序_取系统版本信息():
    "返回示例: Windows 10 (build 18363)，64位版"
    return wx.GetOsDescription()

@组件_异常检测
def 程序_取系统用户名():
    "返回示例: Administrator"
    return wx.GetUserName()

@组件_异常检测
def 程序_系统是否64位():
    "返回True程序运行所在的操作系统是否为64位。"
    return wx.IsPlatform64Bit()

@组件_异常检测
def 程序_打开指定网址或目录(地址):
    "可以打开电脑目录或使用默认浏览器打开指定网址"
    return wx.LaunchDefaultBrowser(地址)

@组件_异常检测
def 程序_打开指定网址(url):
    import webbrowser
    return webbrowser.open(url)

@组件_异常检测
def 程序_取鼠标坐标():
    "返回鼠标坐标(x,y)"
    return wx.GetMouseState().GetPosition()

@组件_异常检测
def 程序_鼠标侧键1是否按下():
    "返回当前应用程序对象,判断鼠标侧边附加的按键是否按下"
    return wx.GetMouseState().Aux1IsDown()

@组件_异常检测
def 程序_鼠标侧键2是否按下():
    "返回True或False,判断鼠标侧边附加的按键是否按下"
    return wx.GetMouseState().Aux2IsDown()

@组件_异常检测
def 程序_鼠标左键是否按下():
    "返回True或False,判断鼠标左键是否按下"
    return wx.GetMouseState().LeftIsDown()

@组件_异常检测
def 程序_鼠标中键是否按下():
    "返回True或False,判断鼠标中键是否按下"
    return wx.GetMouseState().MiddleIsDown()

@组件_异常检测
def 程序_鼠标右键是否按下():
    "返回True或False,判断鼠标右键是否按下"
    return wx.GetMouseState().RightIsDown()

@组件_异常检测
def 程序_取当前进程ID():
    "返回当前程序进程PID"
    return wx.GetProcessId()

@组件_异常检测
def 程序_系统环境是否支持中文():
    "返回True或False    更多环境语言方法参考:https://wxpython.org/Phoenix/docs/html/wx.Language.enumeration.html#wx-language"
    return wx.GetLocale().IsAvailable(wx.LANGUAGE_CHINESE)

@组件_异常检测
def 程序_取环境语言名称():
    "返回示例: Chinese (Simplified)    更多环境语言方法参考:https://wxpython.org/Phoenix/docs/html/wx.Locale.html#wx-locale"
    return wx.GetLocale().GetLocale()

@组件_异常检测
def 程序_取环境语言缩写():
    "返回实力: zh_CN    更多环境语言方法参考:https://wxpython.org/Phoenix/docs/html/wx.Locale.html#wx-locale"
    return wx.GetLocale().GetName()

@组件_异常检测
def 程序_系统是否已激活():
    "返回True或False,不太确定是不是这个命令,获取更多电脑描述信息参考:https://wxpython.org/Phoenix/docs/html/wx.VersionInfo.html#wx-versioninfo"
    return wx.GetLibraryVersionInfo().HasCopyright()

@组件_异常检测
def 程序_执行Dos(命令):
    '运行cmd内的命令,只返回True跟False'
    return wx.Shell(命令)

@组件_异常检测
def 组件_信息框(提示="",标题="提示",类型=0,父窗口=None):
    """
    类型:
    0.无图标信息框
    1.带取消键普通信息框
    2.带是/否键普通信息框
    3.带帮助键普通信息框
    4.带红色错误图标信息框
    5.带黄色感叹标题信息框
    6.带盾牌(类似权限验证)图标信息框

    返回值:2.是   4.确定   8.否   16.取消/关闭   4096.帮助
    """
    字典 = {0:262144,1:16,2:10,3:4096,4:512,5:256,6:524288}
    return wx.MessageBox(提示,标题,字典[类型],父窗口)


@组件_异常检测
def 组件_提示信息框(内容):
    "弹出一个带蓝色反向感叹号图标的信息框"
    return wx.LogMessage(内容)

@组件_异常检测
def 组件_警告信息框(内容):
    "弹出一个带黄色三角形感叹号图标的信息框"
    return wx.LogWarning(内容)

@组件_异常检测
def 组件_报错信息框(内容):
    "弹出一个带红叉图标的信息框"
    return wx.LogError(内容)

@组件_异常检测
def 组件_文件选择器(标题="请选择文件",初始路径="",默认文件名="",过滤器="所有文件|*.*",父窗口=None):
    "选择文件后返回完整文件路径,没选择返回空文本,可添加参数,flags(标识),parent(父窗口),x,y"
    return wx.FileSelector(标题, 初始路径,默认文件名, wildcard=过滤器)

@组件_异常检测
def 组件_保存文件对话框(提示="",后缀="*",默认文件名="",父窗口=None):
    "设置文件后返回完整文件路径,没选择返回空文本"
    return wx.SaveFileSelector(提示, 后缀,默认文件名, 父窗口)

@组件_异常检测
def 组件_目录选择器(提示="",初始路径="",父窗口=None):
    "选择目录后返回完整路径,没选择返回空文本,返回示例:c:\\user"
    return wx.DirSelector(message=提示, default_path=初始路径,parent=父窗口)

@组件_异常检测
def 组件_颜色选择器(初始颜色=None,标题="请选择颜色",父窗口=None):
    "选择颜色后返回颜色值(0,0,0,255),可添加参数,flags(标识),parent(父窗口),x,y"
    return wx.GetColourFromUser(父窗口, 初始颜色,标题)

@组件_异常检测
def 组件_字体选择器(父窗口,默认字体=None,标题="请选择字体"):
    "选择字体后返回字体类型"
    return wx.GetFontFromUser(父窗口,默认字体 if 默认字体 else 父窗口.GetFont(),标题)

@组件_异常检测
def 组件_数值对话框(标题="请设置数值",提示="",参数提示="",默认值=1,最小值=1,最大值=100,父窗口=None):
    "不能在线程里调用,弹出一个设置数值的对话框"
    return wx.GetNumberFromUser(提示, 参数提示, 标题, 默认值, 最小值, 最大值, 父窗口)

@组件_异常检测
def 组件_密码对话框(提示="",标题="请输入密码",默认文本="",父窗口=None):
    "弹出一个文本对话框,输入的内容会被替换成圆点,适合密码等输入使用,确认后返回输入的内容,取消返回空文本"
    return wx.GetPasswordFromUser(message=提示,caption=标题, default_value=默认文本,parent=父窗口)

@组件_异常检测
def 组件_单选列表对话框(提示="",标题="请选择",选择项=['未设置'],初始选中=0,父窗口=None):
    "弹出一个单选列表对话框,选择后返回选中的文本内容,取消返回空,选择项必须是文本型列表,初始选中从0开始"
    return wx.GetSingleChoice(message=提示,caption=标题,choices=选择项,initialSelection=初始选中,parent=父窗口)

@组件_异常检测
def 组件_普通对话框(提示="",标题="请输入",默认文本='',父窗口=None):
    "弹出一个对话框输入文本,确认后返回输入的文本,取消返回空"
    return wx.GetTextFromUser(message=提示,caption=标题,default_value=默认文本,parent=父窗口)

@组件_异常检测
def 组件_气泡提示框(父窗口,提示="",标题="",超时时间=3000,x=0,y=0):
    "弹出一个气泡提示框,默认在组件中间,可通过设置x,y调整"
    气泡 = wx.adv.RichToolTip(标题,提示)
    气泡.SetTimeout(超时时间)
    气泡.ShowFor(父窗口,(0,0,x*2,y*2))

@组件_异常检测
def 组件_系统弹窗(父窗口=None,提示="",标题=""):
    "电脑右下角弹出一个提示框,可以绑定提示框点击事件,详细操作：https://wxpython.org/Phoenix/docs/html/wx.adv.NotificationMessage.html#wx.adv.NotificationMessage.Show"
    提示框 = wx.adv.NotificationMessage(标题,提示,父窗口)
    提示框.Show()





import unittest

from .__init__ import *


class Testenv(unittest.TestCase):

    def test_1(self):
        pass
        data = """
        DOMAIN=example.org
        ADMIN_EMAIL=admin@${DOMAIN}
        ROOT_URL=${DOMAIN}/app
    
        pass=******
        a=pyefun
        b=goefun
        c=${a}-${b}
        """

        data2 = """
        pass=1234567890
        """

        config = 环境变量_从文本中解析(data)
        print(config)
        环境变量_从文本中加载至系统(data)
        print(读环境变量("a"))
        print(读环境变量("c"))
        print(读环境变量("pass"))
        环境变量_从文本中加载至系统(data2)
        print(读环境变量("pass"))

        config = {
            **环境变量_从文本中解析(data),  # 加载文件
            **环境变量_从文本中解析(data2),  # 加密秘钥
            # **环境变量_获取系统所有变量(),  # 加载系统环境变量
        }
        print(config)




import wx.lib.agw.floatspin as floatspin
from .wxControl import *


class 小数微调框(floatspin.FloatSpin, 公用方法):
    pass

    def 取当前数值(self):
        return self.GetDefaultValue().GetValue()

    def 取当前数值2(self):
        return self.GetValue()

    def 取默认数值(self):
        return self.GetDefaultValue()

    def 取显示的位数(self):
        return self.GetDigits()

    def 取字符格式(self):
        return self.GetFormat()

    def 取最大值(self):
        return self.GetMax()

    def 取最小值(self):
        return self.GetMin()

    @组件_异常检测
    def 置最大值(self,最大值):
        return self.SetMax(最大值)

    @组件_异常检测
    def 置最小值(self,最小值):
        return self.SetMin(最小值)

    @组件_异常检测
    def 置数值范围(self,最小值,最大值):
        return self.SetRange(最小值,最大值)

    @组件_异常检测
    def 置数值范围2(self,最小值,最大值):
        return self.SetRangeDontClampValue(最小值,最大值)

    def 是否设置数值范围(self):
        return self.HasRange()

    @组件_异常检测
    def 是否在数值范围内容(self,数值):
        '测试指定数值是否在允许的范围内'
        return self.InRange(数值)

    def 是否已设置默认值(self):
        return self.IsDefaultValue()

    @组件_异常检测
    def 置小数位数(self,位数):
        return self.SetDigits(位数)

    @组件_异常检测
    def 置字符格式(self,格式):
        return self.SetFormat(格式)

    @组件_异常检测
    def 置当前数值(self,数值):
        return self.SetValue(数值)



import unittest

from .stringBase import *

class TestString(unittest.TestCase):

    def test_1(self):
        data = 取文本长度("1234")
        self.assertEqual(data, 4)
        data = 取文本长度("你好")
        self.assertEqual(data, 2)

    def test_2(self):
        data = 取文本左边("1234", 2)
        self.assertEqual(data, "12")
        data = 取文本左边("1234", 10)
        self.assertEqual(data, "1234")

    def test_3(self):
        data = 取文本右边("1234", 2)
        self.assertEqual(data, "34")
        data = 取文本右边("1234", 10)
        self.assertEqual(data, "1234")

        data = 取文本中间("123abc123", 3,3)
        self.assertEqual(data, "abc")
        data = 取文本中间("123abc123", 3,10)
        self.assertEqual(data, "abc123")


    def test_4(self):
        data = 字符(65)
        self.assertEqual(data, "A")
        data = 字符(97)
        self.assertEqual(data, "a")
        data = 字符(66)
        self.assertEqual(data, "B")

    def test_5(self):
        data = 取代码("A")
        self.assertEqual(data, 65)
        data = 取代码("a")
        self.assertEqual(data, 97)

    def test_6(self):
        data = 寻找文本("ABCDEFG", "A")
        self.assertEqual(data, 0)
        data = 寻找文本("ABCDEFG", "B")
        self.assertEqual(data, 1)
        data = 寻找文本("ABCDEFG", "G")
        self.assertEqual(data, 6)
        data = 寻找文本("ABCDEFGA", "A", 1)
        self.assertEqual(data, 7)
        data = 寻找文本("BBBAACCCAAA", "A", 3)
        self.assertEqual(data, 3)
        data = 寻找文本("BBBAACCCAAA", "A", 4)
        self.assertEqual(data, 4)
        data = 寻找文本("BBBAACCCAAA", "A", 5, 7)
        self.assertEqual(data, -1)
        data = 寻找文本("祖国你好", "你好")
        self.assertEqual(data, 2)

    def test_7(self):
        data = 倒找文本("ABCDEFG", "A")
        self.assertEqual(data, 0)
        data = 倒找文本("ABCDEFGA", "A")
        self.assertEqual(data, 7)
        data = 倒找文本("ABCDEFGAa", "A", 2)
        self.assertEqual(data, 7)

    def test_8(self):
        data = 到大写("abc")
        self.assertEqual(data, "ABC")
        data = 到小写("ABC")
        self.assertEqual(data, "abc")

        data = 到全角("abc123456789,./;'[]")
        self.assertEqual(data, "ａｂｃ１２３４５６７８９，．／；＇［］")
        data = 到半角("ａｂｃ１２３４５６７８９，．／；＇［］")
        self.assertEqual(data, "abc123456789,./;'[]")

    def test_9(self):
        data = 删首空("     abc")
        self.assertEqual(data, "abc")
        data = 删尾空("abc     ")
        self.assertEqual(data, "abc")
        data = 删首尾空("     abc     ")
        self.assertEqual(data, "abc")
        data = 删全部空("     a b c     ")
        self.assertEqual(data, "abc")

        data = 子文本替换("1234567890123", "123", "")
        self.assertEqual(data, "4567890")

        data = 子文本替换("1234567890123", "123", "", 1)
        self.assertEqual(data, "4567890123")

        data = 取空白文本(10)
        self.assertEqual(data, "          ")
        data = 取重复文本(10, "a")
        self.assertEqual(data, "aaaaaaaaaa")

        data = 分割文本("1 2 3 4 5 6 7 8 9 0 ", " ")
        self.assertEqual(data, ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0', ''])

        data = 分割文本("1 2 3 4 5 6 7 8 9 0 ", " ",5)
        self.assertEqual(data, ['1', '2', '3', '4', '5', '6 7 8 9 0 '])



import unittest

from .arithmeticOperationBase import *
from .arrayActionBase import *
from .stringBase import *


class TestDirk(unittest.TestCase):

    def test_1(self):
        pass
        # arr = 数组(["a", "b", "c"])
        # arr.加入成员("a")
        arr = 数组()
        for i in range(10):
            arr.加入成员(取随机数(1, 100))

        print(arr.取所有成员())
        arr.排序(reverse=False)
        print(arr.取所有成员())
        arr.排序(reverse=True)
        print(arr.取所有成员())

        arr2 = 数组()
        print("arr2???", arr2.取所有成员())

        for i in range(10):
            arr2.加入成员((取随机数(1, 100), 字符(65 + i)))

        print(arr2.取所有成员())
        arr2.排序(reverse=False)
        print(arr2.取所有成员())
        # arr2.排序(reverse=True,)
        arr2.排序(reverse=True, key=lambda d: d[0])
        print(arr2.取所有成员())

    def test_2(self):
        arr = 数组([1, 2, 3, 4, 5, 0, 0, 0])
        arr.加入成员(6)
        arr.插入成员(0, 2)
        print(arr.取所有成员())

        data = arr.统计成员次数(0)
        print(data)

        data = arr.查找成员(3)
        print(data)
        print(arr.取所有成员())
        data = arr.弹出成员()
        print(data)
        print(arr.取所有成员())
        arr.移除成员(2)
        print(arr.取所有成员())

        arr.翻转()
        print(arr.取所有成员())

        arr.清空()
        print(arr.取所有成员())

    def test_3(self):
        # arr = 数组(["a", "b", "c"])
        # arr.加入成员("a")
        arr = 数组()
        for i in range(10):
            arr.加入成员(取随机数(1, 100))

        print(arr.取所有成员())
        arr.从大到小()
        print(arr.取所有成员())
        arr.从小到大()
        print(arr.取所有成员())

        arr2 = 数组()
        print("arr2???", arr2.取所有成员())

        for i in range(10):
            arr2.加入成员((取随机数(1, 100), 取随机数(1, 100),str(取随机数(1, 100)), 字符(65 + i)))
        print(arr2.取所有成员())
        arr2.从大到小(0)
        print(arr2.取所有成员())
        # arr2.排序(reverse=True,)
        arr2.从小到大(2)
        print(arr2.取所有成员())
        arr2.从小到大(3)
        print(arr2.取所有成员())
        arr2.从大到小(3)
        print(arr2.取所有成员())


import unittest

from .timeUtil import *
from .timeBase import *
from .systemProcessingBase import *


class TestTimeUtil(unittest.TestCase):

    def test_1(self):
        # with 计时器() as t:
        #     延时(1)
        # print(t.取耗时())

        t = 时间统计()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        t.开始()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())

        t = 计时器()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())
        t.开始()
        延时(1.22)
        print(t.取秒())
        print(t.取毫秒())

    def test_2(self):
        for _ in 计时统计(num=1000, verbose=3):
            math.factorial(10000)

    def test_3(self):
        t1 = 计时统计(次数=100, 标签="运行", 取样=20, 显示信息=3, 单位='ms')
        for timer in t1:
            setup_vars = 10000
            with timer:
                延时(0.01 + 取随机数(1, 2)/100)
                math.factorial(setup_vars)
        print('总耗时 = %r' % (t1.取耗时(),))
